(function () {
  'use strict';
  // Source: src/html/scripts/cart/index.js
  angular.module('pine.cart', ['LocalStorageModule']).config([
    'localStorageServiceProvider',
    function (localStorageServiceProvider) {
      localStorageServiceProvider.setStorageType('sessionStorage');
      //    .setNotify(true, true);
      ;
    }
  ]).factory('$cart', [
    '$http',
    '$q',
    'localStorageService',
    function ($http, $q, localStorageService) {
      var cart;
      // public API
      return {
        load: load,
        add: add,
        remove: remove,
        clear: clear,
        save: save,
        items: items
      };
      function load() {
        var cartId;
        var promise;
        // return if already loaded;
        if (cart) {
          return $q.when(cart);
        }
        // check if we already have one (id in local storage)
        cartId = localStorageService.get('cart');
        // if we have no cart id create one
        if (!cartId) {
          return $q.when(initCart());
        }
        // finally if we have an id, get the cart from the server.
        return $http.get('/carts/' + cartId, { 'Accept-Content': 'application/json' }).then(function (res) {
          cart = res.data;
          return cart;
        }, initCart);
      }
      function initCart() {
        // might add some defaults ?
        cart = {
          isNew: true,
          currency: '',
          trip: {
            travellers: [],
            segments: []
          },
          items: [],
          totals: { total: 0 }
        };
        return cart;
      }
      function items(type) {
        if (!type) {
          return cart.items;
        }
        return cart.items.filter(function (item) {
          return (item.type || item['@type']) === type;
        });
      }
      function add(item) {
        cart.items.push(item);
        // make sure we only add ints (you know addition in javascript are approxumate)
        var total = Math.round(cart.totals.total * 100);
        total += item.totals.total * 100;
        cart.totals.total = total / 100;
        return total;
      }
      function remove(item) {
        var index = cart.items.indexOf(item);
        var amount, total;
        if (index > -1) {
          total = cart.totals.total * 100;
          amount = item.totals.total * 100;
          total = total - amount;
          cart.totals.total = total / 100;
          cart.items.splice(index, 1);
          return item;
        }
        return false;
      }
      function clear(type) {
        var cleared, items = cart.items;
        if (type) {
          // mutating a looping array scares me
          // so doing it in two steps
          cleared = items.filter(function (item) {
            return item.type === type || item['@type'] === type;
          });
          for (var i = 0; i < cleared.length; i++) {
            items.splice(cart.items.indexOf(cleared[i]), 1);
          }
        } else {
          cleared = items.slice();
          items.length = 0;
        }
        return cleared;
      }
      function value() {
        return cart;
      }
      function save() {
        var cartId = localStorageService.get('cart'), isNew = !cartId, opts = {
            method: 'POST',
            url: '/carts',
            data: cart
          };
        if (!isNew) {
          // this is a new cart. we should post
          opts.method = 'PUT';
          opts.url += '/' + cartId;
        }
        return $http(opts).then(function (res) {
          // pine-mongo bug : returs array for POST
          if (angular.isArray(res.data)) {
            res.data = res.data[0];
          }
          // WATCHOUT: changinf ref. should we mutate old object (merge) or change ref ?
          cart = res.data;
          // if this was a newly created cart
          // track the id on local storage for later use.
          if (isNew) {
            localStorageService.set('cart', cart._id);
          }
          return cart;
        }, function failed(res) {
          var error = new Error('could not save cart');
          error.status = res.status;
          return error;
        });
      }
    }
  ]);
}());