(function () {
'use strict';
// Source: src/html/scripts/air-sales/air-sales.js
angular.module("air-sales", ["air"]);

angular.module("air", [
	"ui.bootstrap",
	"ui.select",
	"ui.router",
	"ngAnimate",
	"ui.bootstrap.datepicker"
]);

// Source: src/html/scripts/air-sales/config/analytics.js
angular.module('air-sales')
    .run(function ($rootScope, $injector, $log) {

        var $analytics;

        try {
          $analytics = $injector.get('$analytics');
        } catch (e) {
          // Error: [$injector:unpr]
          // just returned, analytics is not loaded

          $log.warn('$analytics not available');
          return;
        }


        $rootScope.$on('airSearchQuery', function ($event, q) {
            var path = '/flights/search/' + q.segments.map(function (seg) {
                return _.str.slugify(seg.from.code || seg.from) + '/' +
                    _.str.slugify(seg.to.code || seg.to);
            }).join('/');

            // time to travel
            path += '/dtd:' + moment(q.segments[0].date).diff(moment(), 'days');

            $log.debug('$analytics.pageTrack', path);
            $analytics.pageTrack(path);
        });

        $rootScope.$on('airOfferDetails', function ($event, quote) {
            var airline = _.str.slugify(quote.trip.segments[0].flights[0].marketer.code),
                airports = quote.trip.segments.map(function (seg) {
                    return _.str.slugify(seg.departure.location.code) + '/' + _.str.slugify(seg.arrival.location.code);
                }).join('/'),
                path = '/flights/offer/' + airline + '/' + airports;

            path += '/dtd:' + moment(quote.trip.segments[0].departure.date).diff(moment(), 'days');
            $log.debug('$analytics.pageTrack', path);
            $analytics.pageTrack(path);
        });

        $rootScope.$on('airBookCartSaved', function($event, cart) {

            var path = '/checkout/';
             var type = cart.items[0]["@type"].toLowerCase();

            switch (type) {
                case 'air':
                    path += 'air';
                    break;
                case 'stayitem':
                    path += 'stay';
                    break;
                case 'omra':
                    path += 'omra';
                    break;
                case 'haj':
                    path += 'hajj';
                    break;
                case 'vo':
                    path += 'voyages-organisés';
                    break;
                default:
                    path += type;

            }

            //$log.info('checkout-update:cart:', path);


            $analytics.pageTrack(path);

        });


    });

// Source: src/html/scripts/air-sales/config/datepicker.js
angular
.module('air')
.config(function (datepickerConfig, datepickerPopupConfig) {
  datepickerConfig.showWeeks = false;
  datepickerConfig.dayFormat = 'd';
  datepickerConfig.startingDay = 1;
  datepickerConfig.minDate = new Date();
  datepickerConfig.maxDate = moment().add(18, 'months').toDate();
  datepickerConfig.maxMode = 'month';
  datepickerPopupConfig.toggleWeeksText = null;
  datepickerPopupConfig.showButtonBar = false;
});

// Source: src/html/scripts/air-sales/config/states.js
angular.module('air').config(function($stateProvider, $urlRouterProvider, settings) {
    $stateProvider.state('default', {
        url: '/?flight&travellers&cabin&flags&airlines&corpid',
        controller: 'air_SearchCtrl',
        templateUrl: 'hermes-air-search',
        resolve: {
            query: ['airQueryCodec', '$stateParams', function(airQueryCodec, $stateParams) {
                return airQueryCodec.fromStateParams($stateParams);
            }]
        }
    }).state('quote', {
        url: '/view/:item?seller',
        reloadOnSearch: true,
        controller: 'air_QuoteCtrl',
        templateUrl: 'hermes-air-quote'
    });
    $urlRouterProvider.otherwise('/');
});

// Source: src/html/scripts/air-sales/controllers/quote.js
angular
    .module('air')
    .controller('air_QuoteCtrl', function($scope, $http, $stateParams, HermesResolver, $timeout, settings, $modal, $log, $window, $rootScope) {
        $log.debug('air_QuoteCtrl:');

        var localAiports = ['AGA', 'AHU', 'CMN', 'CAS', 'ERH', 'EUN', 'FEZ', 'NDR', 'RAK', 'OZZ', 'OUD', 'RBA', 'SFI', 'SII', 'TTA', 'TTU', 'TNG', 'VIL', 'ESU']

        function hasLocalStep(quoteAiroportsCodes) {
            return quoteAiroportsCodes.filter(function(code) {
                return localAiports.indexOf(code) > -1;
            }).length > 0;
        }

        $scope.paxLabels = {
            'adult': 'Adulte',
            'child': 'Enfant',
            'infant': 'Bébé'
        };


        var cabins = {
            'C': 'Business',
            'F': 'First Class',
            'M': 'Eco. Standard',
            'W': 'Eco. Premium',
            'Y': 'Economique'
        };

        var i18n = {
            'Rule Application': 'Application du règlement',
            'Min Stay': 'Séjour minimum',
            'Max Stay': 'séjour amximum',
            'Sales Restrict': 'Restrictions des ventes',
            'Advance Res/tkt': 'Etat des réservations',
            'Flt Application': 'Application des vols',
            'Child Discounts': 'Réduction enfants',
            'Stopovers': 'Escales',
            'Transfers/rtgs': 'Transferts',
            'Surcharges': 'surcharges',
            'Tkt Endorsement': 'Confirmation des dossiers',
            'Penalties': 'Pénailités',
            'Combinability': 'Combinaison',
            'Miscellaneous Data': 'Données diverses',
            'Voluntary Changes': 'Changements volontaires'
        };

        var quotePaths = [{
            field: 'flights',
            path: '.trip.segments[0:]'
        }, {
            field: 'flight',
            path: '.items[0:].product.products'
        }, {
            field: 'travellers',
            path: '.items[0:]'
        }, {
            field: 'travellers',
            path: '.trip'
        }];

        var families = [];

        angular.extend($scope, {
            openConditions: openConditions,
            reprice: priceWithDifferentCarrier,
            toCheckout: toCheckout,
            status: 'loading',
            i18n: i18n,
            cabins: cabins,
            helpers: { hasLocalStep: true },
            getFeesbyclient: getFeesbyclient,
            selectQuotefromfamilies: selectQuotefromfamilies
                // getDevis: getDevis
                //aircrafts:aircrafts
        });
        $rootScope.$on('choose family', function(event, obj) {
            success(obj);

        });

        var endpoint = '/air-quotes/';
        //var endpoint = 'http://labs.fractalite.com:9999';
        //'http://api2.atlasvoyages.com'

        console.log(" state params item : ", $stateParams.item);
        console.log(" state params seller: ", $stateParams.seller);

        if ($stateParams.seller) {
            $http.get(endpoint + $stateParams.item + '?seller=' + $stateParams.seller).success(success).error(failure);
        } else {
            $http.get(endpoint + $stateParams.item).success(success).error(failure);
        }






        function success(data) {
            //  console.log('dataaaaa', data);
            $scope.quotes = data;
            if (Array.isArray(data)) {
                if (data.length > 1) {
                    families = data;
                    $scope.multipleFaresdata = true;
                }
                data = data[0];
            }


            //  console.log(' data from java server : ', data);

            var reflist = HermesResolver.extractReferrables(data);
            HermesResolver.wireObjects(data, reflist, quotePaths);

            $timeout(function() {

                var productsPerFlight = {};
                // build hash map of products per quote
                angular.forEach(data.items, function(item) {
                    if (item['@type'] !== 'air') return;

                    var flight, entry, subProduct;

                    for (var i = 0; i < item.product.products.length; i++) {
                        subProduct = item.product.products[i];
                        flight = subProduct.flight;
                        entry = productsPerFlight[flight.$$hashKey];

                        if (entry === undefined) {
                            productsPerFlight[flight.$$hashKey] = entry = [];
                        }

                        entry.push({
                            paxType: $scope.paxLabels[item.product.paxType],
                            rate: subProduct.code,
                            rateClass: subProduct.class,
                            cabin: cabins[subProduct.cabin],
                            qty: item.quantity,
                            conditions: item.product.airProductPolicies,
                            baggageCount: subProduct.baggageIncluded.quantity,
                            baggageUnit: (subProduct.baggageIncluded.unit === 'N') ? 'baggage/pers.' : 'Kg/pers.'
                        });

                    }

                });
                $scope.productsPerFlight = productsPerFlight;
                $log.debug('productsPerFlight', productsPerFlight);


                var stopovers = {};

                // build a map of stops

                angular.forEach(data.trip.segments, function(seg) {
                    stopovers[seg.$$hashKey] = [];
                    var len = seg.flights.length,
                        i = 1,
                        start, end, duration, h, m, str;

                    while (i < len) {
                        start = seg.flights[i - 1].arrival.date;
                        end = seg.flights[i].departure.date;
                        duration = moment(end).diff(start, 'minutes');
                        h = Math.floor(duration / 60);
                        m = duration % 60;
                        str = '';
                        if (h > 0) {
                            str = h + 'h ';
                        }
                        str += m + 'm';
                        stopovers[seg.$$hashKey].push(str);
                        i++;
                    }


                });

                $scope.stopovers = stopovers;
                $log.debug('stopovers', stopovers);


            });

            $scope.quote = data;
            $scope.$emit('airOfferDetails', data);
            //
            $rootScope.$on('attFees', function(event, obj) {
                console.log('totalFees', obj.Fees.item.price['$t']);
                var feesItem = $scope.quote.items.filter(function(item) {
                    return item['@type'] === 'fee';
                });
                if (feesItem.length > 0) {
                    var index = $scope.quote.items.indexOf(feesItem[0]);
                    $scope.quote.totals.TOTAL = addmoney($scope.quote.totals.TOTAL, feesItem[0].totals.TOTAL, obj.Fees.item.price['$t']);
                    feesItem[0].totals.TOTAL = String(parseInt(obj.Fees.item.price['$t']).toFixed(2)).concat(' MAD');
                    $scope.quote.items[index] = feesItem[0];
                }


            });
            var uniqueAirportsCodes = [];
            data.trip.segments.forEach(function(seg, index, array) {
                var depCode = seg.departure.location.code;
                var ArrivCode = seg.arrival.location.code;
                if (!uniqueAirportsCodes.includes(depCode))
                    uniqueAirportsCodes.push(depCode);
                if (!uniqueAirportsCodes.includes(ArrivCode))
                    uniqueAirportsCodes.push(ArrivCode);
            });

            if (!hasLocalStep(uniqueAirportsCodes)) {
                $scope.helpers.hasLocalStep = false;

            };
            $scope.status = 'done';
        }

        function failure(data, status, headers) {
            $log.error('Failed to get quote', arguments);
            $scope.status = 'error';
            $scope.serverError = data;
        }

        function openConditions(product) {
            $modal.open({
                templateUrl: 'conditions-dialog',
                controller: 'air_ConditionsCtrl',
                size: 'lg',
                resolve: {
                    conditions: function() {
                        return product.airProductPolicies;
                    }
                }
            }).result.then(function(allRead) {
                //console.log('read all ? ', allRead);
                $scope.helpers.conditionsRead = allRead;
            });
        }

        function priceWithDifferentCarrier() {
            $modal.open({
                templateUrl: 'air-reprice-dialog',
                controller: 'air_RepriceCtrl',
                resolve: {
                    quote: function() {
                        return $scope.quote;
                    }
                }
            });
        }

        function toCheckout() {
            var cart = angular.copy($scope.quote);

            var total = cart.totals.TOTAL.split(' ')[0];
            delete cart.totals.total;

            // add enable save
            cart.canSave = true;

            cart.totals.total = $scope.grandTotal || parseFloat(total);

            $scope.status = 'cartSave';

            $http.post('/carts', cart).then(function(res) {
                    // pine-mongo bug : returs array for POST
                    if (angular.isArray(res.data)) {
                        res.data = res.data[0];
                    }

                    $scope.$emit('airBookCartSaved', res.data);

                    // should be settings.links.checkout.replace('cart', res.data._id)
                    $window.location.href = '/ngapps/checkout#/' + res.data._id
                },
                function failed() {
                    $log.error('cart saving failed', arguments);
                    $scope.status = 'error';
                    //$scope.error = 'Probleme de passage à la reservation';

                });

        }
        //get fees by client
        function getFeesbyclient() {
            var modalInstance = $modal.open({
                templateUrl: 'hermes-air-quote-att',
                controller: 'attFeesctrl',
                resolve: {
                    quote: function() {
                        return $scope.quote;
                    }
                }

            });
        }
        //
        function addmoney(op1, op2, op3) {
            var Total;
            var total1 = op1.split(' ')[0];
            var total2 = op2.split(' ')[0];
            var total3 = op3.split('MAD')[0];
            Total = parseInt(total1) + parseInt(total3) - parseInt(total2);
            return String(Total.toFixed(2)).concat(' MAD');
        }

        //
        function selectQuotefromfamilies() {
            console.log('families', families);
            $modal.open({
                templateUrl: 'farefamiliestemplate',
                controller: 'farefamiliesctrl',
                size: 'lg',
                resolve: {
                    families: function() {
                        return families;
                    }
                }
            });
        }
        //
        function getDevis(quote) {
            console.log('quote', quote);
            var filename = "DEVIS";
            var type;
            var segments = [];
            var items = [];
            var total = quote.totals.TOTAL;
            for (var i = 0; i < quote.trip.segments.length; i++) {
                var segment = {};
                segment.dep = quote.trip.segments[i].departure.location.name;
                segment.arr = quote.trip.segments[i].arrival.location.name;
                segments.push(segment);
            }
            console.log('segments', segments);
            for (var i = 0; i < quote.items.length; i++) {
                var item = {};
                item.qunatity = quote.items[i].quantity;
                item.paxtype = quote.items[i].product.paxType;
                item.unit = quote.items[i].product.prices.UNIT;
                item.taxes = quote.items[i].taxes;
                var products = [];
                for (var j = 0; j < quote.items[i].product.products.length; j++) {
                    var product = {};
                    product.baggageIncluded = quote.items[i].product.products[j].baggageIncluded;
                    product.class = quote.items[i].product.products[j].class;
                    product.cabin = quote.items[i].product.products[j].cabin;
                    product.marketer = quote.items[i].product.products[j].flight.marketer;
                    product.operator = quote.items[i].product.products[j].flight.operator;
                    product.equipment = quote.items[i].product.products[j].flight.equipment;
                    product.number = quote.items[i].product.products[j].flight.number;
                    product.departure = quote.items[i].product.products[j].flight.departure;
                    product.arrival = quote.items[i].product.products[j].flight.arrival;
                    products.push(product);
                    item.products = products;
                }
                items.push(item);
            }
            console.log('items', items);
            console.log('get devis');
            var req = {
                url: '/getDevis',
                method: 'GET',
                params: { 'segments': segments, 'items': items, 'total': total }
            };
            $http(req).then(function(response) {
                console.log('response', response);
                var file = new Blob([response.data], {
                    type: 'text/html'
                });
                if (window.navigator.msSaveOrOpenBlob) // IE10+
                    window.navigator.msSaveOrOpenBlob(file, filename);
                else { // Others
                    var a = document.createElement("a"),
                        url = URL.createObjectURL(file);
                    a.href = url;
                    a.download = filename;
                    document.body.appendChild(a);
                    a.click();
                    setTimeout(function() {
                        document.body.removeChild(a);
                        window.URL.revokeObjectURL(url);
                    }, 0);
                }
            }, function errorCallback(response) {
                console.log('error');
            });
        }

    });

angular
    .module('air')
    .controller('air_ConditionsCtrl', function($scope, $modalInstance, conditions, $log) {


        var i18n = {
            'Rule Application': 'Application du règlement',
            'Min Stay': 'Séjour minimum',
            'Max Stay': 'Séjour maximum',
            'Sales Restrict': 'Restrictions de vente',
            'Advance Res/tkt': 'État des réservations',
            'Flt Application': 'Application des vols',
            'Child Discounts': 'Réductions enfants',
            'Stopovers': 'Escales',
            'Transfers/rtgs': 'Transferts',
            'Tkt Endorsement': 'Confirmation des dossiers',
            'Penalties': 'Pénalités',
            'Combinability': 'Combinaison',
            'Miscellaneous Data': 'Données diverses',
            'Voluntary Changes': 'Changements volontaires'
        };



        angular.extend($scope, {
            i18n: i18n,
            conditions: conditions,
            closeConditions: closeConditions,
            showFare: showFare,
            showPolicy: showPolicy,
            read: 1
        });

        showFare(conditions[0]);

        function closeConditions() {
            $modalInstance.close($scope.read > conditions.length);
        }

        function showFare(fare) {
            $scope.currentFare = fare;
            $scope.read++;
            var policy = $scope.currentPolicy;
            if (fare.rules.indexOf(policy) === -1) {
                showPolicy(fare.rules[0]);
            }
        }

        function showPolicy(policy) {
            $scope.currentPolicy = policy;
        }
    });


angular
    .module('air')
    .controller('air_RepriceCtrl', function($scope, settings, $modalInstance, quote, $http, $location) {

        $scope.quote = quote;



        angular.extend($scope, {
            reprice: reprice,
            searchAirlines: searchAirlines,
            loadingAirlines: false,
            close: close,
        });


        function searchAirlines(val) {
            if (val.length < 1) {
                return [];
            }
            $scope.loadingAirlines = true;
            return $http.get(settings.air.airlinesSearch.replace(':term', val)).then(function(res) {
                return res.data || [];
            }).finally(function() {
                $scope.loadingAirlines = false;
            });
        }


        function close() {
            $modalInstance.dismiss();
        }

        function reprice(carrier) {
            if (carrier.code) {
                $location.search('seller', carrier.code);
            }
            // if string...
            else if (carrier.length === 2) {
                $location.search('seller', carrier);
            }
            close();
        }
    });
angular
    .module('air')
    .controller('attFeesctrl', function($scope, $modalInstance, $log, $http, quote, $rootScope) {
        var customer;
        angular.extend($scope, {
            query: {},
            selectCustomer: selectCustomer,
            dismiss: dismiss
        });
        console.log('get att fees');

        $scope.search = function() {
            console.log('query', $scope.query);
            var code = $scope.query.code;
            $scope.status = 'search';
            $scope.customers = null;
            if (!code) {
                $http({
                        url: '/hermes-customers',
                        method: 'GET',
                        params: $scope.query
                    })
                    .success(successCallback)
                    .error(errorCallback);
            } else {
                $http.get('/hermes-customers/' + code)
                    .success(successCallback)
                    .error(errorCallback);
            }
        }

        function successCallback(data) {
            $scope.status = 'found';
            $scope.customers = angular.isArray(data) ? data : [data];
        }

        function errorCallback(status, code) {
            if (code === 404) {
                $scope.status = 'notfound';
            } else {
                $scope.status = 'error';
            }
        }

        function selectCustomer(val) {
            console.log("ok", val);
            $modalInstance.close(val);
            var object = {
                IdCanal: '40',
                BackOfficeId: val.code,
                Currency: 'MAD',
                item: getAttRequest(quote)
            };
            console.log('object', object);
            var req = {
                url: '/GetFees',
                method: 'GET',
                params: { 'attfees': object }
            };

            $http(req).then(function successCallback(response) {
                console.log('responsefromattfees', response.data);
                $rootScope.$broadcast('attFees', response.data);
            }, function errorCallback(response) {
                console.log('error');
            });
        }

        function dismiss() {
            $modalInstance.dismiss('cancel');
        }
        // GET ATT request 


        function getAttRequest(quote) {
            var Item = {};
            var item = {};
            Item.type = 'air';
            item['@'] = Item;
            item.codecompagnie = quote.trip.segments[0].flights[0].marketer.code;
            item.profile = getAttRequestprofiles(quote);
            item.itineraire = getCoupons(quote);

            return item;
        }

        function getAttRequestprofiles(quote) {
            var profiles = [];
            var codecompagnie;
            var Profiles = getPaxes(quote.trip.travellers);
            for (var i = 0; i < Profiles.length; i++) {
                var profile = getProfileobject(Profiles[i].paxtype, Profiles[i].number);
                profiles.push(profile);
            }
            codecompagnie = quote.trip.segments[0].flights[0].marketer.code;
            return profiles;
        }

        function getCoupons(quote) {
            var itineraires = [];
            for (var i = 0; i < quote.trip.segments.length; i++) {
                var Itineraire = {};
                var itineraire = {};
                Itineraire.Libelle = quote.trip.segments[i].departure.location.code + ' ' + quote.trip.segments[i].arrival.location.code;
                itineraire['@'] = Itineraire;
                var Coupons = [];
                for (var j = 0; j < quote.trip.segments[i].flights.length; j++) {
                    var coupon = {};
                    coupon.AeroportDepart = quote.trip.segments[i].flights[j].departure.location.code;
                    coupon.AeroportArrivee = quote.trip.segments[i].flights[j].arrival.location.code;
                    coupon.ClasseTransport = quote.trip.segments[i].seats[j][0].class;

                    Coupons.push(coupon)
                }
                itineraire.coupon = Coupons;
                itineraires.push(itineraire);
            }
            return itineraires;
        }

        function getProfileobject(Pax, number) {
            var profile = {};
            var pax = {};
            var nbr = {};
            pax.class = Pax;
            profile['@'] = pax;
            nbr.nbrpax = number;
            profile.nbrpax = number;
            return profile;
        }

        function getPaxes(travellers) {
            var Profiles = [];
            var paxes = ['adult', 'child', 'infant'];
            for (var i = 0; i < paxes.length; i++) {
                var pax = travellers.filter(function(traveller) {
                    return traveller.classification.pax === paxes[i];
                });
                if (pax.length > 0) {
                    var profile = {};
                    profile.paxtype = paxes[i];
                    profile.number = pax.length;
                    Profiles.push(profile);
                }
            }
            return Profiles;
        }
    });
angular
    .module('air')
    .controller('farefamiliesctrl', function($scope, $modalInstance, $log, $http, families, $rootScope) {
        var familiesTexts = [{
                'marketer': 'TP',
                'families': [{ 'name': 'DISCOUNT', 'Characteristics': ['Baggage à main uniquement', 'Modifications non autorisées', 'Non remboursable'] },
                    { 'name': 'BASIC', 'Characteristics': ['1 bagage en soute', 'Modifications autorisées (service payant)', 'Non remboursable'] },
                    { 'name': 'CLASSIC', 'Characteristics': ['1 bagage en soute', 'Modifications autorisées (service payant)', 'Réservation de sièges standards gratuite', 'Non remboursable'] },
                    { 'name': 'PLUS', 'Characteristics': ['1 bagage en soute', 'Modifications gratuite', 'Remboursable', 'Réservation de sièges gratuites'] },
                    { 'name': 'EXECUTIVE', 'Characteristics': ['2 bagage en soute', 'Modifications autorisées (service payant)', 'Remboursable(service payant)', 'Réservation de sièges gratuite'] },
                    { 'name': 'TOPEXE', 'Characteristics': ['2 bagage en soute', 'Modifications gratuites', 'Remboursable', 'Réservation de sièges gratuite'] }
                ]
            }, {
                'marketer': 'AF',
                'families': [{ 'name': 'LIGHTONE', 'Characteristics': ['Baggage à main uniquement', 'Billet non modifiable', 'Billet non remboursable'] },
                    { 'name': 'LIGHT', 'Characteristics': ['Baggage à main uniquement', 'Billet non modifiable', 'Billet non remboursable'] },
                    { 'name': 'STANDARD', 'Characteristics': ['1 bagage en soute', 'Billet modifiable avec frais', 'Billet non remboursable'] },
                    { 'name': 'FLEX', 'Characteristics': ['1 bagage en soute', 'Billet modifiable sans frais', 'Billet remboursable', 'Siège à l' + "'" + 'avant de la cabine'] }
                ]
            }, {
                'marketer': 'LH',
                'families': [{ 'name': 'LIGHT', 'Characteristics': ['Baggage à main uniquement', 'Billet non modifiable', 'Billet non remboursable'] },
                    { 'name': 'CLASSIC', 'Characteristics': ['1 bagage en soute', 'Billet modifiable avec frais', 'Billet non remboursable'] },
                    { 'name': 'FLEX', 'Characteristics': ['1 bagage en soute', 'Billet modifiable sans frais', 'Billet remboursable avec frais'] },
                    { 'name': 'SAVER', 'Characteristics': ['2 bagage en soute', 'Réservation de sièges'] },
                    { 'name': 'BUSINESS', 'Characteristics': ['2 bagage en soute', 'Réservation de sièges', 'Billet modifiable', 'Billet remboursable'] }
                ]
            }

        ];
        var airline = families[0].trip.segments[0].flights[0].marketer.code; //get the marketer of the first flight,it should be the same for all flights for fare families case.
        var texts = familiesTexts.filter(function(f) {
            return f.marketer === airline;
        });

        $scope.familiesTexts = texts[0];
        $scope.currentSegment = 1;
        console.log('texts', $scope.familiesTexts);
        console.log('families', families);
        var Families = [];
        var selectedFamilies = [];
        var mainSegments = getSegments(families);
        console.log('mainSegment', mainSegments);
        var familyObject = getFamiliesdetails(families);
        console.log('familyObject', familyObject);
        Families = getfamiliesbysegment(mainSegments[0], familyObject); //get all possible families for the first segment, to filter by after
        console.log('Families', Families);
        $scope.familieswithTotals = [];
        if (mainSegments.length === 1) {
            $scope.familieswithTotals = getTotals(families, $scope.selectedFamilies);
            console.log('familieswithTotals', $scope.familieswithTotals);
        }
        console.log('familiesbysegment', Families);

        angular.extend($scope, {
            families: families,
            chooseFamily: chooseFamily,
            segments: mainSegments,
            Families: Families,
            airline: airline,
            removeFamily: removeFamily,
            getTotal: getTotal,
            closemodal: closemodal

        });


        function getfamiliesbysegment(segment, familyObject) {
            var families = [];
            for (var i = 0; i < familyObject.length; i++) {
                var segments = familyObject[i].Segments;

                for (var j = 0; j < segments.length; j++) {
                    if (segment.number === segments[j].number) {
                        var names = getNames(families);
                        //console.log('names', names);
                        if (names.includes(segments[j].familyname) === false) { // a revoir
                            // console.log('index', families.includes(segments[j].familyname));
                            var family = {};
                            family.name = segments[j].familyname;
                            families.push(family);
                        }
                    }
                }
            }
            return families;
        }

        function getNames(Families) {
            var names = [];
            if (Families.length > 0) {
                for (var i = 0; i < Families.length; i++) {
                    names.push(Families[i].name);
                }
            }
            return names;
        }

        function getSegments(families) {
            var Segments = [];
            for (var i = 0; i < families[0].trip.segments.length; i++) {
                var Segment = {};
                Segment.number = i + 1;
                Segment.departure = families[0].trip.segments[i].departure.location.code;
                Segment.arrival = families[0].trip.segments[i].arrival.location.code;
                Segments.push(Segment);
            }
            return Segments;
        }

        function getFamiliesdetails(families) {

            var familyObject = [];
            for (var i = 0; i < families.length; i++) {
                var family = {};
                // family.segmentnumber = i+1;
                var Segments = [];
                for (var j = 0; j < families[i].trip.segments.length; j++) {
                    var Segment = {};
                    Segment.number = j + 1;
                    Segment.familyname = families[i].familiesfare[j];
                    //Segment.departure = families[i].trip.segments[j].departure.location.code;
                    //Segment.arrival = families[i].trip.segments[j].arrival.location.code;
                    //mainSegment = Segment;
                    Segments.push(Segment);
                    family.Segments = Segments;
                }
                familyObject.push(family);
            }
            return familyObject;
        }

        function getPossiblefamilies(name) {
            Families = [];
            for (var i = 0; i < familyObject.length; i++) {
                if (familyObject[i].Segments[0].familyname === name) {
                    var Family = {};
                    Family.name = familyObject[i].Segments[1].familyname;
                    Families.push(Family);
                }
            }
            console.log('PossibleFamilies', Families);
            $scope.Families = Families;
        }

        function matchFamilies(familiesObject, selectedFamilies) {
            var finalFamilies = familiesObject.filter(function(f) {
                return JSON.stringify(getAllfamilies(f)) === JSON.stringify(selectedFamilies);
            });
            console.log('finalFamilies', finalFamilies[0]);
            return finalFamilies[0];
        }

        function getAllfamilies(familyObject) {
            var families = [];
            for (var i = 0; i < familyObject.Segments.length; i++) {
                families.push(familyObject.Segments[i].familyname);
            }
            console.log('allfamilies', families);
            return families;
        }

        function chooseFamily(family) {
            $scope.currentSegment++;
            selectedFamilies.push(family.name);
            $scope.selectedFamilies = selectedFamilies;
            console.log('selectedFamilies', selectedFamilies);
            if (mainSegments.length > 1) {
                getPossiblefamilies(family.name);
            }
            if (mainSegments.length === $scope.currentSegment) {
                $scope.familieswithTotals = getTotals(families, $scope.selectedFamilies);
                console.log('familieswithTotals', $scope.familieswithTotals);
            }
            if ($scope.selectedFamilies.length === mainSegments.length) {
                var finalfamilies = matchFamilies(familyObject, $scope.selectedFamilies);
                var index = familyObject.indexOf(finalfamilies);
                console.log('family', families[index]);
                $rootScope.$broadcast('choose family', families[index]);
                $modalInstance.close();
            }
        }

        function removeFamily() { //this remains valid only for 1 or 2 segments
            console.log('removing family');
            $scope.currentSegment--;
            $scope.Families = getfamiliesbysegment(mainSegments[0], familyObject);
            //$scope.Families = Families;
            //console.log('famililes after removing', Families);
            selectedFamilies = [];
            $scope.selectedFamilies = [];
        }

        function getTotals(families, Family) { //this is remains valid only for 1 or 2 segments
            var familieswithTotals = [];
            if (mainSegments.length === 1) {
                for (var i = 0; i < families.length; i++) {
                    var family = {}
                    var fmls = [];
                    for (var j = 0; j < families[i].familiesfare.length; j++) {
                        fmls.push(families[i].familiesfare[j]);
                    }
                    family.names = fmls;
                    family.total = families[i].totals.TOTAL;
                    familieswithTotals.push(family);
                }
            } else {
                for (var i = 0; i < families.length; i++) {
                    if (families[i].familiesfare[0] === Family[0]) {
                        var family = {}
                        var fmls = [];
                        for (var j = 0; j < families[i].familiesfare.length; j++) {
                            fmls.push(families[i].familiesfare[j]);
                        }
                        family.names = fmls;
                        family.total = families[i].totals.TOTAL;
                        familieswithTotals.push(family);
                    }
                }
            }
            return familieswithTotals;
        }

        function getTotal(family) {
            var fmls = [];
            for (var i = 0; i < $scope.selectedFamilies.length; i++) {
                fmls.push($scope.selectedFamilies[i]);
            }
            fmls.push(family);
            var f = familieswithTotals.filter(function(fm) {
                return fm.names === fmls;
            });
            return f[0].total;
        }

        function closemodal() {
            $modalInstance.close();
        }

    });

// Source: src/html/scripts/air-sales/controllers/search-form.js
angular
	.module("air")
	.controller("air_SearchFormCtrl", function (
		$scope,
		settings,
		$window,
		$http,
		$log,
		airQueryCodec,
		$sce, $document
	) {
		$log.info("air_SearchFormCtrl:airQueryCodec", $scope.query);

		var options = {
			minDeparture: moment()
				.startOf("day")
				.toDate(),
			maxDeparture: moment()
				.add(1, "years")
				.toDate(),
			maxAdults: 9,
			maxChildren: 9,
			maxPax: 9,
			adults: [1, 2, 3, 4, 5, 6, 7, 8, 9],
			children: [0, 1, 2, 3, 4, 5, 7, 8, 9],
			ages: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
			tripTypes: [{
				code: 1,
				label: "Direct flight"
			}, {
				code: 2,
				label: "Round trip"
			}, {
				"code": 3,
				"label": "Multi-segments/open-jaw"
			}]
		};
		/** new form with range date picker configuration */
		$scope.date1 = "";
		$scope.date2 = "";

		$scope.roomDiv = false;
		$scope.roomInput = "Adultes - Enfant ...";

		function rangePickerChange() {
			//console.log('-------- date simple format ------', $scope.date1); 
			var date2 = $scope.date1.split('--');
			$scope.query.segments[0].date = convertDate(date2["0"]);
			$scope.query.segments[1].date = convertDate(date2["1"]);
			console.log(' in range picker fubnction ', $scope.query);
		};

		function simplePickerChange() {
			//console.log('-------- date simple format ------', $scope.date2); 
			$scope.query.segments[0].date = convertDate($scope.date2);
			console.log(' in range picker fubnction ', $scope.query);
		};

		function roomChange() {
			var adulteCount = 0;

			$scope.roomInput = "Adultes " + $scope.query.adults + " ,Enfant " + $scope.query.children;
			$scope.roomDiv = $scope.roomDiv ? false : true;
			//console.log("&&&&&&&&&&&&&&============================================ rooms", $scope.roomInput);
		}
		$document.bind('click', function (event) {
			if (event.target.id !== "inputDivRooms" && event.target.className !== "guests__inputs divRooms ng-scope" && $scope.roomDiv === true && !event.target.classList.contains("divRoomsInputs") && !event.target.nextSibling.classList.contains("divRoomsInputs")) {
				//console.log("&event  ", event, event.target.id, event.target.className, $scope.roomDiv);
				$scope.roomDiv = $scope.roomDiv ? false : true;
				$scope.$apply();
			}


		});

		function convertDate(date) {
			var splitDate = date.split('/');
			var convertedDate = new Date(splitDate[2], splitDate[1] - 1, splitDate[0]);
			return convertedDate.toLocaleDateString("en-US");
		}
		/** end of --------new form with range date picker configuration */
		//
		// , {
		//     "code": 3,
		//     "label": "Multi-segments/open-jaw"
		// }
		var calendars = {};
		var query;
		//
		if ($scope.query && !settings.air.standaloneSearchForm) {

			query = angular.copy($scope.query);
			console.log("------- query if existe with setting air -----", query);
		}
		// else if ($scope.query) {

		// 	query = angular.copy($scope.query);
		// 	console.log("------- query if existe only -----", query);
		// } 
		else {

			query = airQueryCodec.newQuery();
			query.tripType = 0;
			setSegmentCount(2);
			console.log("------- query if not  existe -----", query);
		}



		$scope.$watch("query.children", function (fresh, old) {
			if (fresh !== old) {
				while (query.ages.length < fresh) {
					query.ages.push({});
				}
				while (query.ages.length > fresh) {
					query.ages.pop();
				}
			}
		});

		if (!query.tripType) setSegmentCount(2);

		angular.extend($scope, {
			airports: [],
			airlines: [],
			query: query,
			calendars: calendars,
			setTripType: setSegmentCount,
			addSegment: addSegment,
			removeSegment: removeSegment,
			toggleCalendar: toggleCalendar,
			searchAirports: searchAirports,
			searchAirlines: searchAirlines,
			search: doSearch,
			dateChanged: dateChanged,
			maxSegments: 6,
			options: options,
			// HACK for ui-select wich renders html from different origin (cdnjs/apollo)
			trustAsHtml: $sce.trustAsHtml,
			simplePickerChange: simplePickerChange,
			rangePickerChange: rangePickerChange,
			roomChange: roomChange

		});

		/*
		 * @ngdoc method
		 * @name air_SearchFormCtrl#setSegmentCount
		 * @memberOf ftl.air
		 *
		 *
		 * @description
		 * updates segment array in query
		 */
		function setSegmentCount(tripType) {
			var newCount = tripType;
			if (newCount === query.tripType) return;

			var segments = query.segments;
			var previous,
				i = 0;
			// initial state
			console.log("---- length setSegmentCount ----", query);

			if (query.segments.length === 0) {
				segments.push({
					options: {
						minDate: options.minDeparture,
						minMonth: moment(options.minDeparture).startOf("month"),
						initDate: "query.segments[0].options.minMonth"
					}
				});
			}

			i = segments.length;
			while (segments.length < newCount) {
				previous = query.segments[segments.length - 1];
				segments.push({
					from: previous.to,
					options: {
						minDate: previous.date ? previous.date : previous.minDate,
						minMonth: previous.date ? moment(previous.date).startOf("month") : previous.minMonth,
						initDate: "query.segments[" + i + "].options.minMonth"
					}
				});
				i++;
			}
			if (newCount === 2) {
				segments[1].from = segments[0].to;
				segments[1].to = segments[0].from;
			}


			query.tripType = newCount;


		}

		function addSegment() {
			var i = query.segments.length;

			if (i >= 6) {
				return;
			}

			var previous = query.segments[i - 1];
			query.segments.push({
				from: previous.to,
				options: {
					minDate: previous.date ? previous.date : previous.minDate,
					minMonth: previous.date ? moment(previous.date).startOf("month") : previous.minMonth,
					initDate: "query.segments[" + i + "].options.minMonth"
				}
			});
		}

		function removeSegment(segment) {
			var index = query.segments.indexOf(segment);
			if (index > -1) {
				query.segments.splice(index, 1);
			}
		}

		function searchAirports(val) {
			$scope.airports = [];
			if (val.length < 1) {
				return [];
			}

			return $http
				.get(settings.air.airportsSearch.replace(":term", val))
				.then(function (res) {
					$scope.airports = res.data || [];
					return $scope.airports;
				});
		}

		function searchAirlines(val) {
			$scope.airlines = [];
			if (val.length < 1) {
				return [];
			}

			return $http
				.get(settings.air.airlinesSearch.replace(":term", val))
				.then(function (res) {
					$scope.airlines = res.data || [];
					return $scope.airlines;
				});
		}

		function toggleCalendar($event, cal) {
			var key = "cal" + cal;
			var old = calendars[key];
			for (var k in calendars) {
				calendars[k] = false;
			}
			calendars[key] = !old;
		}

		function doSearch() {
			// make sure last point ===  first
			if (query.tripType === airQueryCodec.ROUND_TRIP) {
				query.segments[1].from = query.segments[0].to;
				query.segments[1].to = query.segments[0].from;
			}

			var qs = airQueryCodec.toStateParams(query);
			if (settings.air && settings.air.standaloneSearchForm) {
				$window.location.href = settings.air.salesApp + "?" + qs;
			} else {
				$window.location.href = settings.links.search + "?" + qs;
			}
		}

		/**
		 * watching date changes to update next segment's min departure date
		 */
		function dateChanged(index) {
			var segments = query.segments,
				segment,
				len = query.segments.length,
				minDate,
				initDate;

			if (!segments[index]) return;

			minDate = query.segments[index].date;

			initDate = moment(minDate)
				.startOf("month")
				.toDate();
			index++;

			while (index < len) {
				segment = segments[index];
				segment.options = segments.options || {
					initDate: "$parent.$parent.options.minMonth"
				};
				segment.options.minDate = minDate;
				segment.options.minMonth = moment(minDate)
					.startOf("month")
					.toDate();
				if (segment.date && moment(segment.date).isBefore(minDate)) {
					segment.date = minDate;
				}
				index++;
			}
			//console.log('dateChanged', index);
		}
	});

// Source: src/html/scripts/air-sales/controllers/search.js
angular.module("air")

	.controller("air_SearchCtrl", function (
		$scope,
		$stateParams,
		airQueryCodec,
		$http,
		$log,
		$timeout,
		resultsHelper,
		calendarsearchHelper,
		query,
		$state,
		settings,
		$anchorScroll,
		$location
	) {

		var quotesUri = "/air-quotes";
		var apiQuery;
		$scope.queryTemp;
		var retries = 0;
		$scope.paxLabels = {
			adult: "Adulte",
			child: "Enfant",
			infant: "Bébé"
		};

		if ($stateParams && !$scope.query && Object.prototype.toString.apply($stateParams.flight) === "[object Array]") {
			if (!$scope.queryTemp) {
				$scope.queryTemp = {};
			}
			var res = $stateParams.flight[0].split(",");
			$scope.queryTemp.depart = res[0];
			$scope.queryTemp.retour = res[1];
			$scope.queryTemp.datedepart = res[2];
			if ($stateParams.flight[1]) {
				var res2 = $stateParams.flight[1].split(",");
				$scope.queryTemp.dateretour = res2[2];
			}

			// console.log('-----------------------------         query         -----------------------------', $scope.queryTemp);
		}
		if (query) {
			$log.info("air_SearchCtrl:", query);
			$scope.$emit("airSearchQuery", query);
			$scope.context = "search";
			$scope.query = query;
			apiQuery = airQueryCodec.toApi(query);
			//console.log(apiQuery);
			$scope.viewState = {
				name: "hermes-air-search-messages",
				status: "searching"
			};
			search();
		} else {
			$scope.status = "blank";
		}

		var stopOptions = [{
			count: 0,
			label: "direct"
		}, {
			count: 1,
			label: "1 escale"
		}, {
			count: 2,
			label: "2 escales"
		}, {
			count: 3,
			label: "3 escales"
		}, {
			count: 4,
			label: "plus de 3 escales"
		}];

		$scope.filterQuery = {};
		//
		// scope (public) methods
		//
		angular.extend($scope, {
			chooseItinerary: chooseItinerary,
			changeItinerary: changeItinerary,
			changeflexibleItinerary: changeflexibleItinerary,
			changeCheapestitinerary: changeCheapestitinerary,
			doQuote: doQuote,
			toggleFilter: toggleFilter,
			resetFilters: resetFilters,
			loaditineraries: loaditineraries,
			filterQuery: {
				applied: []
			},
			cabinNames: {
				C: "Business",
				F: "First",
				M: "Eco Std.",
				W: "Eco Prem.",
				Y: "Economique"
			},
			retryWith: retryWith
		});

		$scope.isArray = angular.isArray;

		function doQuote(link) {
			$state.go("quote", {
				item: encodeURI(link)
			});
		}

		function toggleFilter(name, value) {
			var applied = $scope.filterQuery.applied;
			if ($scope.filterQuery[name] !== value) {
				$scope.filterQuery[name] = value;
				$scope.filterQuery.highlight = name;
				applied.push(name);
			} else delete $scope.filterQuery[name];

			if (!value) {
				applied.splice(applied.indexOf(name), 1);
			}

			var filtered = filterItineraries($scope.itineraries, $scope.filterQuery);

			$scope.helpers.filtered = filtered.length;
		}

		function retryWith(params) {
			for (var param in params) {
				$location.search(param, parmams[param]);
			}
		}

		function search() {
			console.log("in search function ", query);
			$http
				.get(quotesUri + "?" + apiQuery)
				.success(function (data, status) {
					if (status === 202 && retries < 40) {
						if (retries < 40) retries++;
						$timeout(search, 1000);
						return;
					} else if (status === 202) {
						// and retries >= 40
						$scope.viewState.status = "timeout";
						return;
					}
					// reset stuff
					retries = 0;
					if (status === 200) {
						onSearchResults(data);
					} else if (status === 205) {
						$scope.viewState.status = "nocontent";
					} else {
						console.log("error", data);
						$scope.viewState.status = "error";
						$scope.error = data;
						// $scope.viewState.errorMessage = ?
					}
				})
				.error(function (data) {
					$scope.viewState.status = "error";
					$scope.error = data;
				});
		}

		function onSearchResults(data) {
			var segmentCount = $stateParams.flight.length;

			$scope.viewState.status = "done";

			$scope.allQuotes = JSOG.decode(data);
			// console.log('onSearchResults', $scope.allQuotes);

			if (query.flags.flexible && query.segments.length === 2) {
				var depdayquery = $scope.query.segments[0].date;
				var arrdayquery = $scope.query.segments[1].date;
				$scope.LowFare = calendarsearchHelper.getlowestfare($scope.allQuotes);
				$scope.arrdates = calendarsearchHelper.getCalendardates($scope.allQuotes, depdayquery, arrdayquery).arrival.day;
				$scope.depdates = calendarsearchHelper.getCalendardates($scope.allQuotes, depdayquery, arrdayquery).departure.day;
				$scope.calendardeparturdays = calendarsearchHelper.getCalendardates($scope.allQuotes, depdayquery, arrdayquery).departure.textdate;
				$scope.calendararrivaldays = calendarsearchHelper.getCalendardates($scope.allQuotes, depdayquery, arrdayquery).arrival.textdate;
				$scope.calendartotals = calendarsearchHelper.getCalendarFaresandAirlines($scope.allQuotes, $scope.depdates, $scope.arrdates).calendartotals;
				$scope.calendarAirlines = calendarsearchHelper.getCalendarFaresandAirlines($scope.allQuotes, $scope.depdates, $scope.arrdates).CalendarAirlines;
			}


			angular.forEach($scope.allQuotes, processQuoteAvailability);

			$scope.segmentCount = segmentCount;
			$scope.chosenItineraries = [];
			$scope.helpers = {};

			if (segmentCount === 2) {
				$scope.helpers.segmentNames = ["Aller", "Retour"];
				$scope.helpers.theSegmentNames = ["d'aller", "du retour"];
			}

			promptSegment(0);
			//
			//var Allitineraries = $scope.itineraries;
			//var AllAirlines = $scope.filters.airline;
			// console.log('AllAirlines', AllAirlines);
			//console.log('Allitineraries', Allitineraries);
			if (query.segments.length < 3 && !query.flags.flexible) {
				$scope.Cheapestitineraries = getChepeastitinerarybymarketer($scope.itineraries, $scope.filters.airline).sort(function (a, b) {
					return a.total - b.total;
				});
			}
			// console.log('CHEAPESTITINERARIES', $scope.Cheapestitineraries);
		}

		function chooseItinerary(itinerary) {
			console.log('cchooseItinerary ', itinerary);
			var segmentIndex = $scope.currentSegment;
			$scope.chosenItineraries[segmentIndex] = itinerary;
			promptSegment(segmentIndex + 1);
		}



		function attachbaggagetochosenitinerary(selectedQuote) {
			selectedQuote.items.forEach(function (item) {
				for (var k = 0; k < item.product.components.length; k++) {
					item.product.components[k].flight.baggage =
						item.product.components[k].baggageIncluded;
				}
			});
		}

		function fixbaggage(itinerary) {
			itinerary.cheapestQuote.items.forEach(function (item) {
				for (var i = 0; i < item.product.components.length; i++) {
					var baggagesincluded = [];
					item.product.components[i].baggageIncluded.unit =
						item.product.components[i].baggageIncluded.quantity < 9 ? "baggage/pers." : "Kg/pers.";
					if (item.product.components[i].baggageIncluded.indexbag == 2) {
						baggagesincluded.push(item.product.components[i].baggageIncluded);
						item.product.components[i].flight.baggages = baggagesincluded;
					}
				}
			});
		}

		function resetFilters() {
			$scope.filterQuery = {
				applied: []
			};
			$scope.filteredItineraries = filterItineraries(
				$scope.itineraries,
				$scope.filterQuery
			);
			$scope.helpers.filtered = $scope.filteredItineraries.length;
		}




		$scope.getCheapestQuote = function (i) {
			$scope.searchBymarketer = true;
			toggleFilter('airline', i);
			var marketeritineraries = [];
			var i = 0;
			while (i < query.segments.length) {
				marketeritineraries = $scope.itineraries.filter(function (itinerary) {
					return itinerary.hide !== true;
				});
				chooseItinerary(marketeritineraries[0]);
				i++;
			}
		}

		/**
		 * sets up scope so as to prompt the user for the first segment without
		 * a choice, starting at the given index.
		 * given index 0 and chosenItineraries = [choice, null]
		 * => will prompt for index 1
		 * given index 1 and chosenItineraries = [choice, null]
		 * => will prompt for index 1
		 * given index 3 and chosenItineraries = [choice, choice]
		 * => will set status as finished choice
		 *
		 * @scope-requires {allQuotes, chosenItineraries}
		 * @scope-mutates {finished, chosenItineraries, currentSegment}
		 */
		function promptSegment(index) {
			//console.log('promptSegment ' + index);
			$log.debug("chosenItineraries ", $scope.chosenItineraries);
			var allQuotes = $scope.allQuotes;
			var chosenItineraries = $scope.chosenItineraries;

			// whil chosen[index] not empty
			while (chosenItineraries[index]) {
				index++;
			}
			// if choice already made for all segments
			if (index >= allQuotes[0].trip.segments.length) {
				$scope.finished = true;
				$scope.currentSegment = -1;

				// console.log(" chosen itineraries : ", chosenItineraries);
				// theorically we should only get one matching quote for each combination of itineraries
				var matches = resultsHelper.matchTrip(allQuotes, chosenItineraries);
				$log.debug("matching quote", matches);

				if (matches.length > 1) {
					throw new Error("Expected to be only one quote matching a trip");
				}
				// uri is the id for 'quote' ui-state
				matches[0].uri =
					matches[0].uri || matches[0].links.self.replace("/air-quotes/", "");
				$scope.selectedQuote = matches[0];
				// console.log(" selected Quote : ", $scope.selectedQuote);
				var selectedrec = $scope.selectedQuote.items[0].product.components;
				for (var i = 0; i < selectedrec.length; i++) {
					if (selectedrec[i].baggageIncluded != null) {
						selectedrec[i].baggageIncluded.unit =
							selectedrec[i].baggageIncluded.quantity < 9 ? "baggage/pers." : "Kg/pers.";
					}
				}
				// console.log(" selected Quote : ", $scope.selectedQuote);
				if (query.segments.length < 3 && !query.flags.flexible) {
					attachbaggagetochosenitinerary($scope.selectedQuote);
				}
			} else {
				$scope.selectedQuote = null;
				$scope.currentSegment = index;
				var viewData = resultsHelper.buildResultViewData(
					allQuotes,
					index,
					chosenItineraries
				);
				$scope.itineraries = viewData.itineraries;

				if (query.flags.flexible && query.segments.length === 2) {
					$scope.itineraries = calendarsearchHelper.getItinerariesLowestsfare($scope.itineraries);
					$scope.calendardepartureitineraries = calendarsearchHelper.sortitineraries(viewData.itineraries, $scope.depdates);
				}
				$scope.filteredItineraries = viewData.itineraries.slice(0);
				//$scope.itineraries = productsPersFlight(viewData.itineraries);
				//console.log(' $scope.itineraries', $scope.itineraries);
				// console.log('fulldata', viewData);
				$scope.filters = viewData.filters;
				$scope.finished = false;
				$scope.helpers.filtered = $scope.filteredItineraries.length;
				if (query.segments.length < 3 && !query.flags.flexible) {
					for (var i = 0; i < $scope.itineraries.length; i++) {
						fixbaggage($scope.itineraries[i]);
					}
				}
				// scrollTo('prompt')
			}
		}

		function productsPersFlight(itineraries) {
			var itineraryLists = [];

			var productsPerFlight = {};
			// build hash map of products per quote
			angular.forEach(itineraries, function (item) {
				// console.log('productsPersFlight==', item)

				itineraryLists.push(item);
				if (item["@type"] !== "air-itineraty") return;
				// console.log('productsPerFlight')
				var flight, entry;

				for (var i = 0; i < item.flights.length; i++) {
					flight = item.flights[i];
					entry = productsPerFlight[flight.$$hashKey];

					if (entry === undefined) {
						productsPerFlight[flight.$$hashKey] = entry = [];
					}

					entry.push({
						paxType: $scope.paxLabels[flight.paxType],
						qty: flight.quantity,
						baggageCount: flight.baggage.quantity,
						baggageUnit: flight.baggage.unit === "N" ? "baggage/pers." : "Kg/pers."
					});
				}

				itineraryLists.push(item);
			});

			$scope.productsPerFlight = productsPerFlight;
			$log.debug("productsPerFlight", productsPerFlight);
			return itineraryLists;
		}

		function changeItinerary(index) {
			$scope.chosenItineraries[index] = null;
			promptSegment(index);
		}

		function changeflexibleItinerary(index) {
			$scope.chosenItineraries[index] = null;
			$scope.chosenItineraries[index + 1] = null;
			promptSegment(index);
		}
		//same as changeflexibleItinerary
		function changeCheapestitinerary(index) {
			$scope.searchBymarketer = false;
			$scope.chosenItineraries[index] = null;
			$scope.chosenItineraries[index + 1] = null;
			promptSegment(0);
			resetFilters();
		}

		function filterItineraries(itineraries, query) {
			return itineraries.filter(function (itin) {
				if (query.airline && !contains(itin.airlines, query.airline)) {
					itin.hide = true;
					return false;
				}

				if (query.stops && query.stops.value !== itin.stops.length) {
					itin.hide = true;
					return false;
				}

				if (
					query.departure &&
					!contains(query.departure.dates, itin.departure.date)
				) {
					itin.hide = true;
					return false;
				}

				if (
					query.arrival &&
					!contains(query.arrival.dates, itin.arrival.date)
				) {
					itin.hide = true;
					return false;
				}

				if (query.duration && itin._tags.duration !== query.duration.value) {
					itin.hide = true;
					return false;
				}

				itin.hide = false;
				return true;
			});
		}

		function processQuoteAvailability(quote) {
			quote.available = quote.items.reduce(function (sum, item) {
				return sum + item.product.availability;
			}, 0);
			quote.available = Math.min(9, quote.available);
		}

		function contains(haystack, needle) {
			return haystack.indexOf(needle) > -1;
		}


		function loaditineraries(itinerary, index1, index2) {
			console.log('itinerary', itinerary);
			var depdayquery = $scope.query.segments[0].date;
			var arrdayquery = $scope.query.segments[1].date;
			if (itinerary == null) {
				return;
			}
			var found = false;
			var itineraries = [];
			if (!Array.isArray(itinerary)) {
				itineraries.push(itinerary)
			} else {
				itineraries = itinerary;
			}
			var quotes = $scope.allQuotes;
			var dates = calendarsearchHelper.getCalendardates(quotes, depdayquery, arrdayquery);
			var selectedquotes = [];
			for (var i = 0; i < quotes.length; i++) {
				if (found == false) {
					if (index1 == dates.departure.day.indexOf(new Date(quotes[i].trip.segments[0].departure.date).getDate())) {
						if (index2 == dates.arrival.day.indexOf(new Date(quotes[i].trip.segments[1].departure.date).getDate())) {
							var index = compareitineraries(quotes[i].trip.segments[0], itineraries)
							selectedquotes.push(itineraries[index]);
							selectedquotes.push(quotes[i].trip.segments[1]);
							found = true;
						}
					}
				}
			}
			if (found == false) {
				return;
			}
			//
			var airlines = calendarsearchHelper.getAirlines(selectedquotes[1].flights);
			var marketer = calendarsearchHelper.getOneairline(airlines.Airlines, airlines.codes);
			selectedquotes[1].marketer = marketer;
			console.log('selectedquotes', selectedquotes);
			//
			chooseflexibleItinerary(selectedquotes);
		}

		function compareitineraries(itinerary, itineraries) {
			var index;
			for (var i = 0; i < itineraries.length; i++) {
				if (itineraries[i].departure.date == itinerary.departure.date && itineraries[i].airlines[0].code == itinerary.airlines[0].code)
					index = i;
			}
			return index;
		}

		function chooseflexibleItinerary(itineraries) {
			console.log('itineraries', itineraries);
			var segmentIndex = $scope.currentSegment;
			$scope.chosenItineraries = itineraries;
			promptSegment(segmentIndex + 2);

		}
		//
		function getChepeastitinerarybymarketer(itineraries, allairlines) {
			var Cheapestitineraries = [];
			for (var j = 0; j < allairlines.length; j++) {
				var itinerariesbymarketer = itineraries.filter(function (itinerary) {
					return itinerary.marketer.code === allairlines[j].code;
				});
				if (itinerariesbymarketer.length === 0) {
					itinerariesbymarketer = itineraries.filter(function (itinerary) {
						return checkmultiairlines(allairlines[j].name, itinerary) === true;
					})
				}
				var sorteditin = itinerariesbymarketer.sort(function (a, b) {
					return a.cheapestQuote.totals.total - b.cheapestQuote.totals.total;
				});
				if (sorteditin.length > 0) {
					var cheapestitinerary = {};
					cheapestitinerary.airline = allairlines[j];
					cheapestitinerary.total = sorteditin[0].cheapestQuote.totals.total;
					Cheapestitineraries.push(cheapestitinerary);
				}
			}
			return Cheapestitineraries;

		}
		// 
		function checkmultiairlines(airlinename, itinerary) {
			var index = false;
			if (itinerary.marketer.code === 'multi') {
				if (itinerary.marketer.name.includes(airlinename) === true) {
					index = true;
				}
			}
			return index;
		}


	});

// Source: src/html/scripts/air-sales/directives/loading-bar.min.js
/*! 
 * angular-loading-bar v0.6.0
 * https://chieffancypants.github.io/angular-loading-bar
 * Copyright (c) 2014 Wes Cruver
 * License: MIT
 */
!function(){"use strict";angular.module("angular-loading-bar",["cfp.loadingBarInterceptor"]),angular.module("chieffancypants.loadingBar",["cfp.loadingBarInterceptor"]),angular.module("cfp.loadingBarInterceptor",["cfp.loadingBar"]).config(["$httpProvider",function(a){var b=["$q","$cacheFactory","$timeout","$rootScope","cfpLoadingBar",function(b,c,d,e,f){function g(){d.cancel(i),f.complete(),k=0,j=0}function h(b){var d,e=c.get("$http"),f=a.defaults;!b.cache&&!f.cache||b.cache===!1||"GET"!==b.method&&"JSONP"!==b.method||(d=angular.isObject(b.cache)?b.cache:angular.isObject(f.cache)?f.cache:e);var g=void 0!==d?void 0!==d.get(b.url):!1;return void 0!==b.cached&&g!==b.cached?b.cached:(b.cached=g,g)}var i,j=0,k=0,l=f.latencyThreshold;return{request:function(a){return a.ignoreLoadingBar||h(a)||(e.$broadcast("cfpLoadingBar:loading",{url:a.url}),0===j&&(i=d(function(){f.start()},l)),j++,f.set(k/j)),a},response:function(a){return a.config.ignoreLoadingBar||h(a.config)||(k++,e.$broadcast("cfpLoadingBar:loaded",{url:a.config.url,result:a}),k>=j?g():f.set(k/j)),a},responseError:function(a){return a.config.ignoreLoadingBar||h(a.config)||(k++,e.$broadcast("cfpLoadingBar:loaded",{url:a.config.url,result:a}),k>=j?g():f.set(k/j)),b.reject(a)}}}];a.interceptors.push(b)}]),angular.module("cfp.loadingBar",[]).provider("cfpLoadingBar",function(){this.includeSpinner=!0,this.includeBar=!0,this.latencyThreshold=100,this.startSize=.02,this.parentSelector="body",this.spinnerTemplate='<div id="loading-bar-spinner"><div class="spinner-icon"></div></div>',this.loadingBarTemplate='<div id="loading-bar"><div class="bar"><div class="peg"></div></div></div>',this.$get=["$injector","$document","$timeout","$rootScope",function(a,b,c,d){function e(){k||(k=a.get("$animate"));var e=b.find(n).eq(0);c.cancel(m),r||(d.$broadcast("cfpLoadingBar:started"),r=!0,u&&k.enter(o,e),t&&k.enter(q,e),f(v))}function f(a){if(r){var b=100*a+"%";p.css("width",b),s=a,c.cancel(l),l=c(function(){g()},250)}}function g(){if(!(h()>=1)){var a=0,b=h();a=b>=0&&.25>b?(3*Math.random()+3)/100:b>=.25&&.65>b?3*Math.random()/100:b>=.65&&.9>b?2*Math.random()/100:b>=.9&&.99>b?.005:0;var c=h()+a;f(c)}}function h(){return s}function i(){s=0,r=!1}function j(){k||(k=a.get("$animate")),d.$broadcast("cfpLoadingBar:completed"),f(1),c.cancel(m),m=c(function(){var a=k.leave(o,i);a&&a.then&&a.then(i),k.leave(q)},500)}var k,l,m,n=this.parentSelector,o=angular.element(this.loadingBarTemplate),p=o.find("div").eq(0),q=angular.element(this.spinnerTemplate),r=!1,s=0,t=this.includeSpinner,u=this.includeBar,v=this.startSize;return{start:e,set:f,status:h,inc:g,complete:j,includeSpinner:this.includeSpinner,latencyThreshold:this.latencyThreshold,parentSelector:this.parentSelector,startSize:this.startSize}}]})}();
// Source: src/html/scripts/air-sales/directives/ui-datepicker-ext.js

angular.module('ui.bootstrap.datepicker')
.controller('DatepickerController', ['$scope', '$attrs', '$parse', '$interpolate', '$timeout', '$log', 'dateFilter', 'datepickerConfig', function($scope, $attrs, $parse, $interpolate, $timeout, $log, dateFilter, datepickerConfig) {
  var self = this,
      ngModelCtrl = { $setViewValue: angular.noop }; // nullModelCtrl;

  // Modes chain
  this.modes = ['day', 'month', 'year'];


  // Configuration attributes
  angular.forEach(['formatDay', 'formatMonth', 'formatYear', 'formatDayHeader', 'formatDayTitle', 'formatMonthTitle',
                   'minMode', 'maxMode', 'showWeeks', 'startingDay', 'yearRange'], function( key, index ) {
    self[key] = angular.isDefined($attrs[key]) ? (index < 8 ? $interpolate($attrs[key])($scope.$parent) : $scope.$parent.$eval($attrs[key])) : datepickerConfig[key];
  });

  // Watchable date attributes
  angular.forEach(['minDate', 'maxDate'], function( key ) {
    if ( $attrs[key] ) {
      $scope.$parent.$watch($parse($attrs[key]), function(value) {
        self[key] = value ? new Date(value) : null;
        self.refreshView();
      });
    } else {
      self[key] = datepickerConfig[key] ? new Date(datepickerConfig[key]) : null;
    }
  });

  if (angular.isDefined($attrs.initDate)) {
    //console.log($attrs.initDate, $parse($attrs.initDate));
    $scope.$parent.$watch($parse('$parent.$parent.' + $attrs.initDate), function(value) {
        //console.log('initdare ' + value);
      if (!ngModelCtrl.$modelValue) {
        self.activeDate = value ? new Date(value) : new Date();
        self.refreshView();
      }
    });
  }

  $scope.datepickerMode = $scope.datepickerMode || datepickerConfig.datepickerMode;
  $scope.uniqueId = 'datepicker-' + $scope.$id + '-' + Math.floor(Math.random() * 10000);
  this.activeDate = angular.isDefined($attrs.initDate) ? $scope.$parent.$eval($attrs.initDate) : new Date();

  $scope.isActive = function(dateObject) {
    if (self.compare(dateObject.date, self.activeDate) === 0) {
      $scope.activeDateId = dateObject.uid;
      return true;
    }
    return false;
  };

  this.init = function( ngModelCtrl_ ) {
    ngModelCtrl = ngModelCtrl_;

    ngModelCtrl.$render = function() {
      self.render();
    };
  };

  this.render = function() {
    if ( ngModelCtrl.$modelValue ) {
      var date = new Date( ngModelCtrl.$modelValue ),
          isValid = !isNaN(date);

      if ( isValid ) {
        this.activeDate = date;
      } else {
        $log.error('Datepicker directive: "ng-model" value must be a Date object, a number of milliseconds since 01.01.1970 or a string representing an RFC2822 or ISO 8601 date.');
      }
      ngModelCtrl.$setValidity('date', isValid);
    }
    this.refreshView();
  };

  this.refreshView = function() {
    if ( this.element ) {
      this._refreshView();

      var date = ngModelCtrl.$modelValue ? new Date(ngModelCtrl.$modelValue) : null;
      ngModelCtrl.$setValidity('date-disabled', !date || (this.element && !this.isDisabled(date)));
    }
  };

  this.createDateObject = function(date, format) {
    var model = ngModelCtrl.$modelValue ? new Date(ngModelCtrl.$modelValue) : null;
    return {
      date: date,
      label: dateFilter(date, format),
      selected: model && this.compare(date, model) === 0,
      disabled: this.isDisabled(date),
      current: this.compare(date, new Date()) === 0,
      classes: ($attrs.dateClasses && $scope.dateClasses({date: date, mode: $scope.datepickerMode})) || ''
    };
  };

  this.isDisabled = function( date ) {
    return ((this.minDate && this.compare(date, this.minDate) < 0) || (this.maxDate && this.compare(date, this.maxDate) > 0) || ($attrs.dateDisabled && $scope.dateDisabled({date: date, mode: $scope.datepickerMode})));
  };

  // Split array into smaller arrays
  this.split = function(arr, size) {
    var arrays = [];
    while (arr.length > 0) {
      arrays.push(arr.splice(0, size));
    }
    return arrays;
  };

  $scope.select = function( date ) {
    if ( $scope.datepickerMode === self.minMode ) {
      var dt = ngModelCtrl.$modelValue ? new Date( ngModelCtrl.$modelValue ) : new Date(0, 0, 0, 0, 0, 0, 0);
      dt.setFullYear( date.getFullYear(), date.getMonth(), date.getDate() );
      ngModelCtrl.$setViewValue( dt );
      ngModelCtrl.$render();
    } else {
      self.activeDate = date;
      $scope.datepickerMode = self.modes[ self.modes.indexOf( $scope.datepickerMode ) - 1 ];
    }
  };

  $scope.move = function( direction ) {
    var year = self.activeDate.getFullYear() + direction * (self.step.years || 0),
        month = self.activeDate.getMonth() + direction * (self.step.months || 0);
    self.activeDate.setFullYear(year, month, 1);
    self.refreshView();
  };

  $scope.toggleMode = function( direction ) {
    direction = direction || 1;

    if (($scope.datepickerMode === self.maxMode && direction === 1) || ($scope.datepickerMode === self.minMode && direction === -1)) {
      return;
    }

    $scope.datepickerMode = self.modes[ self.modes.indexOf( $scope.datepickerMode ) + direction ];
  };

  // Key event mapper
  $scope.keys = { 13:'enter', 32:'space', 33:'pageup', 34:'pagedown', 35:'end', 36:'home', 37:'left', 38:'up', 39:'right', 40:'down' };

  var focusElement = function() {
    $timeout(function() {
      self.element[0].focus();
    }, 0 , false);
  };

  // Listen for focus requests from popup directive
  $scope.$on('datepicker.focus', focusElement);

  $scope.keydown = function( evt ) {
    var key = $scope.keys[evt.which];

    if ( !key || evt.shiftKey || evt.altKey ) {
      return;
    }

    evt.preventDefault();
    evt.stopPropagation();

    if (key === 'enter' || key === 'space') {
      if ( self.isDisabled(self.activeDate)) {
        return; // do nothing
      }
      $scope.select(self.activeDate);
      focusElement();
    } else if (evt.ctrlKey && (key === 'up' || key === 'down')) {
      $scope.toggleMode(key === 'up' ? 1 : -1);
      focusElement();
    } else {
      self.handleKeyDown(key, evt);
      self.refreshView();
    }
  };
}]);

// Source: src/html/scripts/air-sales/filters/duration.js
angular.module('air')

.filter(
    'duration',
    function() {
      return function (str, format) {
        if (!str) { return ''; }

        var dur = moment.isDuration(str) ? str : moment.duration(str);
        var hours = Math.floor(dur.asHours());
        var minutes = dur.minutes();
        var out;
        if (format === 'compact') {
          out = hours + 'h ';
          out += (minutes < 10 ) ? '0' + minutes : minutes;
          return out + 'm';
        }

        out =  hours;
        out += (hours > 1) ? ' heures ' : 'heure';

        if (minutes > 0) {
          out += minutes + ' min.';
        }
        //return dur.humanize();
        return out;
      };
    });

// Source: src/html/scripts/air-sales/filters/flight-date.js

// Source: src/html/scripts/air-sales/filters/flight-stops.js

// Source: src/html/scripts/air-sales/services/calendarsearch-helper.js
angular
    .module('air')
    .factory('calendarsearchHelper', function() {

        //get the lowest fare
        function getlowestfare(quotes) {
            var lowestfare;
            quotes.sort(function(a, b) {
                return a.totals.total - b.totals.total;
            });
            lowestfare = quotes[0].totals.total;
            return lowestfare;
        }

        //getcalendardates
        function getCalendardates(quotes, departure, arrival) {
            var dates = {};
            var depday = [];
            var arrday = [];
            var depmonth = [];
            var arrmonth = [];
            var dep = {};
            var arr = {};
            var deptext = [];
            var arrtext = [];
            depday = getDaysandMonthsarray(departure).daysarray;
            arrday = getDaysandMonthsarray(arrival).daysarray;
            depmonth = getDaysandMonthsarray(departure).montharray;
            arrmonth = getDaysandMonthsarray(arrival).montharray;
            dep.day = depday;
            //  console.log('depday', depday);
            dep.month = depmonth;
            //   console.log('depmonth', depmonth);
            arr.day = arrday;
            arr.month = arrmonth;
            for (var i = 0; i < arr.day.length; i++) {
                deptext[i] = dep.day[i] + ' ' + formatDate(dep.month[i]);
                arrtext[i] = arr.day[i] + ' ' + formatDate(arr.month[i]);
            }
            arr.textdate = arrtext;
            dep.textdate = deptext;
            dates.departure = dep;
            dates.arrival = arr;
            //   console.log('departure', dates.departure);
            return dates;
        }

        //getdaysandmonths

        function getDaysandMonthsarray(date) {
            // console.log('date', date);
            var dayquery = parseInt(date.split("-")[2]);
            var monthquery = parseInt(date.split("-")[1]);
            var yearquery = parseInt(date.split("-")[0]);
            var monthindex = daysInMonth(monthquery, yearquery);
            var monthindex2 = daysInMonth(monthquery - 1, yearquery);
            var daysarray = [];
            var montharray = [];
            daysarray[3] = dayquery;
            montharray[3] = monthquery;
            for (var j = 2; j >= 0; j--) {
                if (dayquery < 4) {
                    if (dayquery - 1 == 0) {
                        daysarray[j] = monthindex2;
                        if (monthquery === 1) {
                            montharray[j] = 12;
                            monthquery = 12;
                        } else {
                            montharray[j] = monthquery - 1;
                            monthquery = monthquery - 1;
                        }
                    } else {
                        daysarray[j] = dayquery - 1;

                        montharray[j] = monthquery;
                    }
                    dayquery = daysarray[j];
                } else {
                    daysarray[j] = dayquery - 1;
                    dayquery = daysarray[j];
                    montharray[j] = monthquery;
                }
            }
            dayquery = parseInt(date.split("-")[2]);
            monthquery = parseInt(date.split("-")[1]);
            for (var i = 4; i < 7; i++) {
                if (dayquery == monthindex) {
                    daysarray[i] = 1;
                    dayquery = 1;
                    if (monthquery === 12) {
                        montharray[i] = 1;
                        monthquery = 1;
                    } else {
                        montharray[i] = monthquery + 1;
                        monthquery = monthquery + 1;
                    }
                } else {
                    daysarray[i] = dayquery + 1;
                    dayquery = dayquery + 1;
                    montharray[i] = monthquery;
                }
            }
            return {
                daysarray: daysarray,
                montharray: montharray
            };
        }


        function daysInMonth(month, year) {
            return new Date(year, month, 0).getDate();
        }

        function formatDate(index) {
            var monthNames = [
                "Jan", "Fév", "Mar",
                "Avr", "Mai", "Jui", "Jul",
                "Aou", "Sep", "Oct",
                "Nov", "Déc"
            ];



            return monthNames[index - 1];

        }
        //
        function getCalendarFaresandAirlines(quotes, departuredates, arrivaldates) {
            quotes = addDaysindextoquotes(quotes);
            var calendartotals = [];
            var CalendarAirlines = [];

            for (var j = 0; j < arrivaldates.length; j++) {
                var selectedquotes = quotes.filter(function(quote) {
                    return quote.arrivaleday === arrivaldates[j];
                });
                if (selectedquotes.length !== 0) {
                    selectedquotes = sortquotes(selectedquotes, departuredates);
                    var totals = [];
                    var calendarAirlines = [];
                    for (var i = 0; i < selectedquotes.length; i++) {
                        if (isEmpty(selectedquotes[i])) {
                            totals.push(0);
                            calendarAirlines.push(null);
                        } else {
                            totals.push(selectedquotes[i].totals.total)
                            var airlines = getAirlines(selectedquotes[i].trip.segments[0].flights);
                            calendarAirlines.push(getOneairline(airlines.Airlines, airlines.codes));
                        }
                    }
                    calendartotals[j] = totals;
                    //console.log('calendartotals', calendartotals[j], j);
                    CalendarAirlines[j] = calendarAirlines;
                    //console.log('CalendarAirlines', CalendarAirlines[j], j);
                }
            }
            return {
                calendartotals: calendartotals,
                CalendarAirlines: CalendarAirlines
            };
        }
        //add daysindex to quotes
        function addDaysindextoquotes(quotes) {
            for (var i = 0; i < quotes.length; i++) {
                quotes[i].departureday = new Date(quotes[i].trip.segments[0].departure.date).getDate();
                quotes[i].arrivaleday = new Date(quotes[i].trip.segments[1].departure.date).getDate();
            }
            return quotes;
        }
        //sort quotes by departuredays
        function sortquotes(quotes, departuredates) {
            var sortedquotes = [];
            for (var i = 0; i < quotes.length; i++) {
                var index = departuredates.indexOf(quotes[i].departureday);
                sortedquotes[index] = quotes[i];
            }
            return sortedquotes;
        }
        //get airlines
        function getAirlines(flights) {
            var codes = getallAirlinescode(flights);
            var Airlines = [];
            for (var i = 0; i < flights.length; i++) {
                var airline = {};
                airline.code = flights[i].marketer.code;
                airline.name = flights[i].marketer.name;
                Airlines.push(airline);
            }
            return { Airlines: Airlines, codes: codes };
        }

        function getOneairline(airlines, codes) {
            var airline = {};
            var names = [];
            if (codes.length > 1) {
                airline.code = 'multi';
                for (var i = 0; i < airlines.length; i++) {
                    names.push(airlines[i].name);
                }
                airline.name = getAirlinesname(names);
            } else {
                airline.code = airlines[0].code;
                airline.name = airlines[0].name;
            }

            return airline;
        }

        function getAirlinesname(names) {
            var name;
            if (names.length === 1) {
                name = name[0];
            } else {
                name = names.join(",");
            }
            return name;
        }

        function getallAirlinescode(flights) {
            var codes = [];
            for (var i = 0; i < flights.length; i++) {
                if (codes.includes(flights[i].marketer.code) === false) {
                    codes.push(flights[i].marketer.code);
                }
            }
            return codes;
        }

        function isEmpty(obj) {
            for (var key in obj) {
                if (obj.hasOwnProperty(key))
                    return false;
            }
            return true;
        }

        function getItinerariesLowestsfare(itineraries) {
            var Lowestitinerariesfare = [];
            Lowestitinerariesfare.push(itineraries[0]);
            for (var i = 0; i < itineraries.length; i++) {
                if (checkitinerary(Lowestitinerariesfare, itineraries[i]).found == false) {
                    Lowestitinerariesfare.push(itineraries[i]);
                } else {
                    var itin = checkitinerary(Lowestitinerariesfare, itineraries[i]).itinerary;
                    //console.log(quote);

                    // if (itineraries[i].cheapestQuote.totals.total < itin.cheapestQuote.totals.total) {
                    //     var index = affordableitineraries.indexOf(itin);
                    //     if (index > -1) {
                    //         affordableitineraries.splice(index, 1);
                    //     }
                    //     affordableitineraries.push(itineraries[i]);
                    // }


                }
            }
            Lowestitinerariesfare.splice(0, 1);
            return Lowestitinerariesfare;
        }
        //
        function checkitinerary(itineraries, itinerary) {
            var found = false;
            var itinfound;
            for (var i = 0; i < itineraries.length; i++) {
                if (new Date(itineraries[i].departure.date).getDate() === new Date(itinerary.departure.date).getDate()) {
                    if (itineraries[i].arrival.date !== itinerary.arrival.date) {
                        found = true;
                        itinfound = itineraries[i];
                        break;
                    }
                }
            }

            return {
                found: found,
                itinerary: itinfound
            }
        }
        // sort itineraries by calendar standards
        function sortitineraries(itineraries, depdates) {
            //console.log('itineraries', itineraries);
            //console.log('depdates', depdates);
            var daysindex = [];
            var sorteditineraries = [];
            if (itineraries.length > 7) {
                var newitins = getitinerariesbydate(itineraries);
                // console.log('newitins', newitins);
                for (var i = 0; i < newitins.length; i++) {
                    daysindex[i] = new Date(newitins[i][0].departure.date).getDate();
                    for (var k = 0; k < newitins[i].length; k++) {
                        newitins[i][k].departure.indexday = new Date(newitins[i][k].departure.date).getDate();
                    }

                }
                if (daysindex.includes(1)) {
                    for (var j = 0; j < daysindex.length; j++) {

                        if (daysindex[j] > 0 && daysindex[j] < 7)
                            daysindex[j] = daysindex[j] + 40;
                    }
                }
                daysindex.sort(function(a, b) {
                    return a - b;
                });
                for (var j = 0; j < daysindex.length; j++) {
                    for (var j = 0; j < daysindex.length; j++) {

                        if (daysindex[j] > 40)
                            daysindex[j] = daysindex[j] - 40;
                    }
                }
                for (var k = 0; k < newitins.length; k++) {
                    var t1 = newitins[k][0].departure.indexday;
                    var t2 = daysindex.indexOf(t1);
                    sorteditineraries[t2] = newitins[k];
                }

                if (sorteditineraries.length < 7) {
                    var Sorted = [];
                    var l = 7 - sorteditineraries.length;
                    for (var i = 0; i < l.length; i++) {
                        Sorted[i] = null;
                    }
                    for (var j = 0; j < sorteditineraries.length; j++) {
                        Sorted[j + l] = sorteditineraries[j];
                    }
                    sorteditineraries = Sorted;
                    //console.log('sorteditineraries', sorteditineraries);
                }
            } else {
                for (var i = 0; i < itineraries.length; i++) {
                    //console.log('iii', i);
                    itineraries[i].departure.indexday = new Date(itineraries[i].departure.date).getDate();
                    //console.log('indexday', itineraries[i].departure.indexday);
                    daysindex[i] = new Date(itineraries[i].departure.date).getDate();
                }
                if (daysindex.includes(1)) {
                    for (var j = 0; j < daysindex.length; j++) {

                        if (daysindex[j] > 0 && daysindex[j] < 7)
                            daysindex[j] = daysindex[j] + 40;
                    }
                }
                daysindex.sort(function(a, b) {
                    return a - b;
                });
                for (var j = 0; j < daysindex.length; j++) {
                    for (var j = 0; j < daysindex.length; j++) {

                        if (daysindex[j] > 40)
                            daysindex[j] = daysindex[j] - 40;
                    }
                }
                for (var k = 0; k < itineraries.length; k++) {
                    var t1 = itineraries[k].departure.indexday;
                    var t2 = daysindex.indexOf(t1);
                    sorteditineraries[t2] = itineraries[k];
                }
                for (var i = 0; i < sorteditineraries.length; i++) {
                    if (sorteditineraries[i] === undefined) {
                        sorteditineraries.splice(i, 1);
                    }
                }
                //console.log('sorteditinerariesAFTER', sorteditineraries);
                if (sorteditineraries.length < 7) {
                    var Sorted = [];
                    var l = 7 - sorteditineraries.length;
                    for (var i = 0; i < l.length; i++) {
                        Sorted[i] = null;
                    }
                    for (var j = 0; j < sorteditineraries.length; j++) {
                        Sorted[j + l] = sorteditineraries[j];
                    }
                    sorteditineraries = Sorted;
                }
                // console.log('sorteditineraries', sorteditineraries);

            }
            return sorteditineraries;
        }

        //get itineraries by dates
        function getitinerariesbydate(itineraries) {
            var itins = [];
            var itinerary;
            while (itineraries.length > 0) {
                //console.log('itineraries.length', itineraries.length);
                for (var i = 0; i < itineraries.length; i++) {
                    if (itineraries[i] == null) {
                        itineraries.splice(i, 1);

                    }
                }
                // if (itineraries[0] == null) {
                //     itineraries.splice(0, 1);
                // }
                // if (itineraries.length == 0) {
                //     return itins;
                // }
                itinerary = itineraries[0];
                if (itinerary == undefined) {
                    return itins;
                }
                var newitins = [];
                newitins.push(itinerary);

                for (var i = 1; i < itineraries.length; i++) {
                    if (new Date(itinerary.departure.date).getDate() == new Date(itineraries[i].departure.date).getDate()) {
                        newitins.push(itineraries[i]);
                        itineraries[i] = null;
                    }
                }
                itineraries[0] = null;
                for (var i = 0; i < itineraries.length; i++) {
                    if (itineraries[i] == null)
                        itineraries.splice(i, 1);
                }
                itins.push(newitins);
            }
            return itins;
        }

        //
        function getfullcalendaredepartures(itineraries, dates) {
            //  console.log('itineraries', itineraries);
            //console.log('dates', dates);
            var newitineraries = [];

            var k = 0;

            for (var i = 0; i < dates.length; i++) {
                if (Array.isArray(itineraries[i])) {
                    if (k == itineraries.length) {
                        newitineraries[i] = null;
                    } else {

                        if (dates[i] == new Date(itineraries[k][0].departure.date).getDate()) {

                            newitineraries[i] = itineraries[k][0];
                            k++;
                        } else {
                            newitineraries[i] = null;
                        }
                    }
                    //   console.log('newitineraries', newitineraries)
                } else {
                    if (k == itineraries.length) {
                        newitineraries[i] = null;
                    } else {

                        if (dates[i] == new Date(itineraries[k].departure.date).getDate()) {

                            newitineraries[i] = itineraries[k];
                            k++;
                        } else {
                            newitineraries[i] = null;
                        }
                    }
                }

            }

            return newitineraries;
        }

        //
        //Compare itineraries

        return {
            getlowestfare: getlowestfare,
            getCalendardates: getCalendardates,
            getCalendarFaresandAirlines: getCalendarFaresandAirlines,
            getItinerariesLowestsfare: getItinerariesLowestsfare,
            sortitineraries: sortitineraries,
            getAirlines: getAirlines,
            getOneairline: getOneairline
        };
    });

// Source: src/html/scripts/air-sales/services/hermes1-resover.js
angular
    .module('air')
.factory(
    'HermesResolver', [

        function () {



            var quotePaths = [{
                field: "flights",
                path: ".trip.segments[0:]"
            }, {
                field: "flight",
                path: ".items[0:].product.products"
            }, {
                field: "travellers",
                path: ".items[0:]"
            }, {
                field: "travellers",
                path: ".trip"
            }];

            var unwireQuotePaths = [{
                field: "flight",
                path: ".items[0:].product.products"
            }, {
                field: "travellers",
                path: ".items[0:]"
            }, ];

            var service = {};
            service.extractReferrables = function (thing) {
                var reflist = {};
                var nodes = JSPath.apply('..*{."@id"}', thing);
                for (var i = 0; i < nodes.length; i++) {
                    reflist[nodes[i]['@id']] = nodes[i];
                }
                //console.log("refs", reflist);
                return reflist;
            };

            service.wireObjects = function (thing, reflist, paths) {
                for (var i = 0; i < paths.length; i++) {
                    var field = paths[i].field;
                    var nodes = JSPath.apply(paths[i].path, thing);
                    //console.log(paths[i].path, nodes.length, nodes);
                    for (var j = 0; j < nodes.length; j++) {
                        var node = nodes[j];
                        if (angular.isNumber(node[field])) {
                            var ref = node[field];
                            node[field] = reflist[ref];
                            //  console.log("wired simple ",node[field]);

                        } else if (angular.isArray(node[field])) {
                            for (var k = 0; k < node[field].length; k++) {
                                var ref = node[field][k];
                                //console.log("wiring array item ",node[field][k]);
                                if (angular.isNumber(node[field][k])) {
                                    node[field][k] = reflist[ref];
                                    //console.log("wired "+ field + k + "  items", node[field]);
                                }
                            }
                        }
                    }
                }
            };

            service.unwireObjects = function (thing, paths) {
                //console.log("unwiring objects");
                for (var i = 0; i < paths.length; i++) {
                    var field = paths[i].field;
                    var nodes = JSPath.apply(paths[i].path, thing);

                    //console.log(paths[i].path, nodes.length, nodes);
                    for (var j = 0; j < nodes.length; j++) {
                        var node = nodes[j];
                        if (angular.isArray(node[field])) {
                            for (var k = 0; k < node[field].length; k++) {
                                var id = node[field][k]['@id'];
                                node[field][k] = id;
                            }
                        } else if (angular.isObject(node[field])) {
                            var id = node[field]['@id'];
                            node[field] = id;
                        }
                    }
                }
                //console.log("UNWIRED", thing);

                return thing;

            };


            service.wireField = function (parent, field, reflist) {
                if (angular.isNumber(parent[field])) {
                    var ref = parent[field];
                    parent[field] = reflist[ref];
                } else if (angular.isArray(parent[field])) {
                    var array = parent[field];
                    for (var i = 0; i < array.length; i++) {
                        array[i] = reflist[array[i]];
                    }
                }
            };

            return service;

        }


    ]
);
// Source: src/html/scripts/air-sales/services/html-money.js

angular.module("air")
.filter('htmlMoney', function(numberFilter, $sce) {

  return function (val) {
    if (!val) return '';

    var tmp = val.split(" ");
    var amount = tmp[0].split('.');
    return $sce.trustAsHtml(numberFilter(amount[0]) + "<sup>." +  amount[1] + "</sup> " + tmp[1] );
  }

});

// Source: src/html/scripts/air-sales/services/jsog.js
/* jshint eqeqeq: false, unused: true, undef:false */
(function() {
    var JSOG, isArray;

    JSOG = {};

    //nextId = 1;

    isArray = Array.isArray || function(obj) {
        return Object.prototype.toString.call(obj) === '[object Array]';
    };

    // https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Global_Objects/Array/indexOf#Polyfill
    if (!Array.prototype.indexOf) {
        Array.prototype.indexOf = function(searchElement, fromIndex) {

            var k;
            if (this == null) {
                throw new TypeError('"this" is null or not defined');
            }

            var O = Object(this),
                len = O.length >>> 0;

            if (len === 0) {
                return -1;
            }
            var n = +fromIndex || 0;

            if (Math.abs(n) === Infinity) {
                n = 0;
            }

            if (n >= len) {
                return -1;
            }

            k = Math.max(n >= 0 ? n : len - Math.abs(n), 0);

            // 9. Repeat, while k < len
            while (k < len) {
                var kValue;
                if (k in O && O[k] === searchElement) {
                    return k;
                }
                k++;
            }
            return -1;
        };
    }

    JSOG.ID_KEY = '$id';
    JSOG.REF_KEY = '$ref';


    JSOG.cleanUp = function(sofar, used) {
        for (var id in sofar) {
            if (used.indexOf(id) === -1) {
                delete sofar[id][JSOG.ID_KEY];
            }
        }
    };

    JSOG.encode = function(original) {
        var doEncode, idOf, cleanUp, sofar, used, nextId = 1;

        sofar = {};
        used = [];
        idOf = function(obj) {
            if (!obj.__jsogObjectId) {
                obj.__jsogObjectId = '' + (nextId++);
            }
            return obj.__jsogObjectId;
        };

        doEncode = function(original) {
            var encodeArray, encodeObject;

            encodeObject = function(original) {
                var id, key, result, value;

                id = idOf(original);
                //console.log("============================id==========================", id)
                if (sofar[id]) {
                    if (used.indexOf(id) === -1) used.push(id);
                    result = {};
                    result[JSOG.REF_KEY] = id;
                    return result;
                }
                result = sofar[id] = {};
                result[JSOG.ID_KEY] = id;
                for (key in original) {
                    value = original[key];
                    if (key !== '__jsogObjectId') {
                        result[key] = doEncode(value);
                    }
                }
                return result;
            };
            encodeArray = function(original) {
                var val;

                return (function() {
                    var _i, _len, _results;

                    _results = [];
                    for (_i = 0, _len = original.length; _i < _len; _i++) {
                        val = original[_i];
                        _results.push(doEncode(val));
                    }
                    return _results;
                })();
            };
            if (original == null) {
                return original;
            } else if (isArray(original)) {
                return encodeArray(original);
            } else if (typeof original === 'object') {
                return encodeObject(original);
            } else {
                return original;
            }
        };
        var result = doEncode(original)
        JSOG.cleanUp(sofar, used);
        return result;
    };

    JSOG.decode = function(encoded) {
        var doDecode, found;

        found = {};
        doDecode = function(encoded) {
            var decodeArray, decodeObject;

            decodeObject = function(encoded) {
                var id, key, ref, result, value;

                ref = encoded.$ref;
                if (ref != null) {
                    ref = ref.toString();
                }
                if (ref) {
                    return found[ref];
                }
                result = {};
                id = encoded.$id;
                if (id != null) {
                    id = id.toString();
                }
                if (id) {
                    found[id] = result;
                }
                for (key in encoded) {
                    value = encoded[key];
                    if (key !== '$id') {
                        result[key] = doDecode(value);
                    }
                }
                return result;
            };
            decodeArray = function(encoded) {
                var value;

                return (function() {
                    var _i, _len, _results;

                    _results = [];
                    for (_i = 0, _len = encoded.length; _i < _len; _i++) {
                        value = encoded[_i];
                        _results.push(doDecode(value));
                    }
                    return _results;
                })();
            };
            if (encoded == null) {
                return encoded;
            } else if (isArray(encoded)) {
                return decodeArray(encoded);
            } else if (typeof encoded === 'object') {
                return decodeObject(encoded);
            } else {
                return encoded;
            }
        };
        return doDecode(encoded);
    };

    JSOG.stringify = function(obj) {
        return JSON.stringify(JSOG.encode(obj));
    };

    JSOG.parse = function(str) {
        return JSOG.decode(JSON.parse(str));
    };

    if ((typeof module !== "undefined" && module !== null) && module.exports) {
        module.exports = JSOG;
    }

    if (typeof window !== "undefined" && window !== null) {
        window.JSOG = JSOG;
    }

    if (typeof define === 'function' && define.amd) {
        define('JSOG', [], function() {
            return JSOG;
        });
    }

    return JSOG;

}).call(this);

// Source: src/html/scripts/air-sales/services/query-parser.js
angular
    .module('air')
    .factory('airQueryCodec',
        /**
         * @ngdoc service
         * @name air.airQueryCodec
         * @memberOf ftl.air
         *
         * @requires $rootScope
         *
         * @description
         * convert air queries from/to stateParmas, and to api Query
         */
        function($http, settings, $log) {
            var knownFlags = [
                'nonstop',
                'direct',
                // No advance purchase fares
                'nap',
                // No penalties fares
                'npe',
                // No restriction fares
                'nr',
                // Refundable fares
                'rf',
                'flexible'
            ];
            var MOMENT_DATE_ISO = 'YYYY-MM-DD';
            var ROUND_TRIP = 2;
            var ONE_WAY = 1;
            var MULTI_SEGMENTS = 3;
            // regexFlight = /^[A-Z]{3},[A-Z]{3},\d{4}-\d{2}-\d{2}$/,
            // this regex stinks
            // regexTravellers = /^\d,\d?\d(,\d?\d){0,5}$/;

            return {
                ROUND_TRIP: ROUND_TRIP,
                ONE_WAY: ONE_WAY,
                MULTI_SEGMENTS: MULTI_SEGMENTS,
                knownFlags: knownFlags,
                newQuery: newQuery,
                fromStateParams: function(encoded) {
                    //console.log('rrrr', encoded);
                    if (!encoded.flight) {
                        return null;
                    }

                    var iataCodes = {};
                    var decoded = newQuery();
                    //console.log('rrrr', decoded);
                    // segments
                    if (!angular.isArray(encoded.flight)) {
                        encoded.flight = [encoded.flight];
                    }
                    decoded.segments = encoded.flight.map(function(flight) {
                        // TO_DO validate regexFlight
                        var bits = flight.split(',');

                        if (!iataCodes[bits[0]]) {
                            iataCodes[bits[0]] = { code: bits[0], name: bits[0] };
                        }
                        if (!iataCodes[bits[1]]) {
                            iataCodes[bits[1]] = { code: bits[1], name: bits[0] };
                        }
                        return {
                            from: iataCodes[bits[0]],
                            to: iataCodes[bits[1]],
                            date: bits[2]
                        };
                    });

                    // linking tripType t segmentLength is a bad idea.
                    // ex: this bugs if its a departure open jaw.
                    decoded.tripType = Math.min(3, decoded.segments.length);
                    // thus...
                    if (decoded.tripType === 2 && decoded.segments[0].from !== decoded.segments[1].to) {
                        decoded.tripType = 3;
                    }

                    // travellers
                    encoded.travellers = encoded.travellers || '1';
                    var travellers = encoded.travellers.split(',');
                    decoded.adults = parseInt(travellers.shift(), 10) || 1;
                    console.log('travellers', travellers);
                    decoded.ages = travellers
                        .map(function(a) {
                            return {
                                v: parseInt(a, 10)
                            };
                        })
                        .filter(function(c) {
                            return c.v >= 0;
                        });
                    decoded.children = decoded.ages.length;

                    // flags
                    if (encoded.flags) {
                        encoded.flags
                            .split(',')
                            .filter(function(value) {
                                return knownFlags.indexOf(value) > -1;
                            })
                            .reduce(function(target, flag) {
                                target.flags[flag] = true;
                                return target;
                            }, decoded);
                    }

                    decoded.cabin = encoded.cabin || '-';
                    decoded.corpid = encoded.corpid;


                    // TODO allow multiple airlines
                    if (encoded.airlines) {
                        var pair = encoded.airlines.split('_');
                        decoded.preferredAirline = { code: pair[0], name: pair[1] };
                    }

                    return $http.get(settings.air.airportsResolve.replace(':codes', Object.keys(iataCodes).join(',')))
                        .then(function(res) {
                            angular.forEach(res.data, function(item) {
                                angular.extend(iataCodes[item.code], item);
                            });
                            $log.debug('decoded query', decoded);
                            return decoded;
                        }, function() {
                            // error just return without resolved names
                            return decoded;
                        });
                },
                toApi: function(query) {
                    var cabin = query.cabin || '-';
                    var out = '';
                    var carriers = query.preferredAirline ? query.preferredAirline.code : 'YY';

                    // ?itinerary=YY..CMNPAR20140619-&itinerary=YY..PARCMN20140628-&flags=direct,flexDates&travellers=2,1,4

                    // exclude VYTB
                    angular.forEach(query.segments, function(segment) {
                        out += '&itinerary=' + carriers + '..' + segment.from.code + segment.to.code + moment(segment.date).format('YYYYMMDD') + cabin;
                    });
                    // Convert travellers.
                    var travellers = query.ages.map(function(age) {
                        return age.v;
                    });
                    travellers.unshift(query.adults);
                    out += '&travellers=' + travellers.join(',');
                    var flags = Object.keys(query.flags)
                        .filter(function(flag) {
                            return query.flags[flag] && knownFlags.indexOf(flag) > -1;
                        }).join(',');

                    if (['M', 'C', 'F'].indexOf(query.cabin) > -1) {
                        out += '&cabin=' + query.cabin;
                    }

                    if (flags) {
                        out += '&flags=' + flags;
                    }

                    if (query.corpid) {
                        out += '&corpid=' + query.corpid;
                    }

                    return out;
                },
                toStateParams: function(decoded) {
                    var encoded = [];
                    console.log('aaaa', decoded);
                    //
                    // Segments
                    //
                    // make sure rountd trip data is set
                    if (decoded.tripType === ROUND_TRIP) {
                        decoded.segments[1].from = decoded.segments[0].to;
                        decoded.segments[1].to = decoded.segments[0].from;
                        // make sure other segments are cut;
                        decoded.segments.length = 2;
                    }
                    // make sure first flight
                    if (decoded.tripType === ONE_WAY) {
                        decoded.segments.length = 1;
                    }
                    // make sure we cuttof garbage.
                    // decoded.segments.length = Math.min(decoded.tripType, decoded.segments.length);

                    angular.forEach(decoded.segments, function(segment) {
                        encoded.push('flight=' +
                            segment.from.code + ',' +
                            segment.to.code + ',' +
                            moment(segment.date).format(MOMENT_DATE_ISO));
                    });

                    //
                    // Travellers
                    //
                    var travellers = (decoded.ages || []).map(function(a) {
                        return a.v;
                    });
                    // add adults in i=0
                    travellers.unshift(decoded.adults);
                    encoded.push('travellers=' + travellers.join(','));

                    //
                    // Cabin
                    //
                    if (decoded.cabin) {
                        encoded.push('&cabin=' + decoded.cabin);
                    }

                    if (decoded.preferredAirline) {
                        encoded.push('&airlines=' + decoded.preferredAirline.code + '_' + decoded.preferredAirline.name);
                    }

                    if (decoded.corpid) {
                        encoded.push('&corpid=' + decoded.corpid);
                    }

                    //
                    // Flags
                    //
                    // console.log('toStateParams flags',decoded.flags);
                    var flags = knownFlags.filter(function(flag) {
                        // console.log('toStateParams flag ',flag,decoded.flags[flag]);

                        // return true if query.flags[flag] === true
                        return decoded.flags[flag];
                    }).join(',');

                    if (flags) {
                        encoded.push('flags=' + flags);
                    }
                    $log.debug('toStateParams', decoded, encoded);

                    return encoded.join('&');
                }
            };

            function newQuery() {
                return {
                    tripType: ROUND_TRIP,
                    segments: [],
                    adults: 1,
                    children: 0,
                    ages: [],
                    cabin: '-',
                    corpid: '',
                    flags: knownFlags.reduce(function(reduced, value) {
                        reduced[value] = false;
                        return reduced;
                    }, {})
                };
            }
        });

// Source: src/html/scripts/air-sales/services/results-helper.js
angular
    .module('air')
    .factory('resultsHelper', function($filter) {


        var settings = {
            dayPeriods: [{
                start: 0,
                value: 4,
                label: 'Nuit (minuit à 5h)'
            }, {
                start: 5,
                value: 11,
                label: 'Matin (5h à 12h)'
            }, {
                start: 12,
                value: 17,
                label: 'Après-midi (12h à 18h)'
            }, {
                start: 18,
                value: 23,
                label: 'Soir (18h à minuit)'
            }]
        };

        var paxLabels = {
            'adult': 'Adulte',
            'child': 'Enfant',
            'infant': 'Bébé'
        };


        return {
            buildResultViewData: buildResultViewData,
            matchTrip: quotesWithItineraries
        };

        /**
         * For a list of quotes and a given segmentIndex.
         * returns an Object {itineraries: array<Itineraries>, filters: Object}
         *
         */
        function buildResultViewData(quotes, segmentIndex, chosenItins) {

            console.log('quotes count', quotes.length);

            var filtered = quotesWithItineraries(quotes, chosenItins),
                result;

            //console.log('filtered count', filtered.length, filtered);

            if (segmentIndex !== undefined) {

                return _uniqueItineraries(filtered, segmentIndex);

            } else {
                throw new Error("Quote view not implemented");
            }



        }



        function quotesWithItineraries(quotes, itineraries) {


            if (itineraries.length === 0) {
                return quotes.slice(0);
            }

            return quotes.filter(function(quote) {
                var i = itineraries.length;
                var segments = quote.trip.segments;
                while (i--) {
                    if (itineraries[i] !== null && segments.indexOf(itineraries[i]) === -1) {
                        return false;
                    }
                }
                return true;
            });

        }





        // ,
        //                 baggageIncluded: {
        //                     unit: (component.baggageIncluded.unit === 'N') ? 'baggage/pers.' : 'Kg/pers.',
        //                     quantity: component.baggageIncluded.quantity
        //                 }




        function unitbaggage(baggages) {
            if (baggages != null) {
                baggages.forEach(function(baggage) {

                    baggage.unit = (baggage.quantity < 9) ? 'baggage/pers.' : 'Kg/pers.';

                })
            }
        }


        /*
         * Given a list of quotes, returns a list of unique itineraries
         * for segment at quotes[*].trip.segment[segmentIndx]
         *
         */
        function _uniqueItineraries(quotes, segmentIndex) {
            var i = 0,
                len = quotes.length,
                uniqueItineraries = [],
                itinerary = {},
                index,
                filters = {
                    airline: [],
                    stops: [],
                    departure: [],
                    arrival: [],
                    duration: []
                        // baggages: []
                };

            for (i = 0; i < quotes.length; i++) {
                itinerary = quotes[i].trip.segments[segmentIndex];

                var Flights = itinerary.flights;
                //console.log('flightsf===', Flights)
                for (var j = 0; j < Flights.length; j++) {
                    var baggages = Flights[j].baggages;
                    unitbaggage(baggages);
                }
                //var diff = (new Date).getTime() - start;
                //console.log('diffff', diff);
                //IncludedBaggageInfos(quotes[i].items, itinerary.flights);

                //console.log('itemsitemsitemsitemsitemsitemsitems', quotes[i].items)
                //console.log('itineraryitineraryitineraryitineraryitineraryitinerary', itinerary)

                // baggageInfos(quotes[i].items, itinerary);

                //var buildIncludedBaggagesMap = IncludedBaggageInfos(quotes[i].items, itinerary.flights);

                // attachIncludedBaggage(itinerary.flights, buildIncludedBaggagesMap);


                if (!contains(uniqueItineraries, itinerary)) {
                    //console.log('itineraryitinerary', itinerary)

                    uniqueItineraries.push(itinerary);
                    processAirlines(itinerary, filters.airline);
                    processStops(itinerary, filters.stops);
                    processTime(itinerary, filters);

                    if (!itinerary.marketer) {
                        var marketers = [];
                        itinerary.flights.forEach(function(f) {
                            if (marketers.indexOf(f.marketer) === -1) {
                                marketers.push(f.marketer)
                            }
                        });
                        if (marketers.length === 1) {
                            itinerary.marketer = marketers[0];
                        } else {
                            itinerary.marketer = {
                                code: 'multi',
                                name: marketers.map(function(m) {
                                    return m.name;
                                }).join(', ')
                            }
                        }
                    }

                }
                itinerary.cheapestQuote = cheapest(itinerary.cheapestQuote, quotes[i]);
                itinerary.corponame = getCorporateTag(quotes[i]);

            }

            filters.airline.sort(function(a, b) {
                return a.name < b.name;
            });

            filters.departure = finalizeFilterTime(filters.departure);
            filters.arrival = finalizeFilterTime(filters.arrival);
            filters.duration = finalizeFilterDuration(filters.duration);
            filters.stops = finalizeFilterStops(filters.stops);
            //console.log('unique itins', uniqueItineraries.length, filters);

            return {
                itineraries: uniqueItineraries,
                filters: filters
            };
        }
        //  setTimeout(_uniqueItineraries, 1500);
        /*
         * search for corporate name
         */


        function getCorporateTag(quote) {
            //console.log('rrr', quote)
            for (var i = quote.items.length - 1; i >= 0; i--) {
                var item = quote.items[i]

                if (item.product.tags) {

                    var tag = item.product.tags.filter(function(tag) {

                        //  if (tag != null) {
                        // return tag.code == 'corporate' ? tag.code : null;
                        if (tag.code == 'corporate' || tag.code == null)
                            return tag;
                        //}
                    });
                    // console.log('eeee', tag);
                    if (tag.length)
                        return tag[0].name;
                }
            }



        }

        function cheapest(quote1, quote2) {
            if (!quote1) return quote2;
            if (!quote2) return quote1;

            return (quote1.totals.total > quote2.totals.total) ? quote2 : quote1;
        }

        function contains(array, item) {
            return array.indexOf(item) > -1;
        }


        function processAirlines(itinerary, sofar) {
            var i = itinerary.flights.length;
            itinerary.airlines = [];

            angular.forEach(itinerary.flights, function(flight) {
                itinerary.airlines.push(flight.marketer);
                if (!contains(sofar, flight.marketer)) {
                    sofar.push(flight.marketer);
                }
                // if (!contains(sofar, flight.operator)) {
                //     sofar.push(flight.operator);
                // }
            });


        }

        function processStops(itinerary, stops) {
            var flights = itinerary.flights;
            var i = 0,
                diff, last = flights.length - 1;

            // first time
            if (!itinerary.stops) {
                itinerary.stops = [];
                while (i < last) {
                    diff = moment.duration(moment(flights[i + 1].departure.date).diff(flights[i].arrival.date));
                    itinerary.stops.push($filter('duration')(diff, 'compact'));
                    //itinerary.stops.push(diff.humanize());
                    i++;
                }
            }

            var filterVal = Math.min(3, itinerary.flights.length - 1);
            if (!contains(stops, filterVal))
                stops.push(filterVal);

        }

        //function processbaggage(itinerary, Baggages) {
        //var bags = itinerary.filghts.baggages;


        //}

        function processTime(itinerary, filters) {
            var departure = itinerary.departure.date,
                arrival = itinerary.arrival.date,
                duration = moment.duration({
                    hours: itinerary._formattedDuration.h,
                    minutes: itinerary._formattedDuration.m,
                }).asHours();

            duration = Math.ceil(moment.duration(itinerary.duration).asHours());

            if (!contains(filters.departure, departure)) {
                filters.departure.push(departure);
            }
            if (!contains(filters.arrival, arrival)) {
                filters.arrival.push(arrival);
            }
            if (!contains(filters.duration, duration)) {
                filters.duration.push(duration);
            }
            itinerary._tags = {
                duration: duration
            };

        }

        function finalizeFilterTime(dates) {

            var values = [];
            angular.forEach(dates, function(date) {

                var hours = moment(date).hours(),
                    periods = settings.dayPeriods,
                    i = periods.length;
                while (i--) {
                    periods[i].dates = periods[i].dates || [];
                    if (periods[i].start <= hours && periods[i].value >= hours) {
                        if (!contains(values, periods[i])) {
                            values.push(periods[i]);
                        }
                        if (!contains(periods[i].dates, date)) {
                            periods[i].dates.push(date);
                        }
                    }
                }

            });

            values.sort(function(a, b) {
                return a - b;
            });


            return values;
        }

        function finalizeFilterStops(stopVals) {
            var i = 0,
                len = stopVals.length,
                out = [];
            stopVals.sort(function(a, b) {
                return a - b;
            });

            while (i < stopVals.length) {
                if (stopVals[i] === 0) {
                    out.push({
                        value: 0,
                        label: 'Vols directs'
                    });
                } else {
                    out.push({
                        value: stopVals[i],
                        label: (stopVals[i] > 1) ? stopVals[i] + ' escales' : 'Une escale'
                    });
                }
                i++;
            }

            return out;
        }

        //function finalizeFilterBags(bags){
        //  var out = [];
        //}


        function finalizeFilterDuration(durations) {
            var out = [];
            var i = 1,
                duration;
            durations.sort(function(a, b) {
                return a - b;
            });

            out.push({
                value: durations[0],
                label: 'Les plus rapides'
            });

            while (i < durations.length) {
                duration = durations[i];
                out.push({
                    value: duration,
                    label: 'Entre ' + (duration - 1) + ' et ' + duration + ' heures'
                });
                i++;
            }
            return out;

        }







    });
}());