(function () {
'use strict';
// Source: src/html/scripts/air-sales/air-form.js
angular.module("air", ["ui.bootstrap", "ui.select", "ui.bootstrap.datepicker"]);

// Source: src/html/scripts/air-sales/config/datepicker.js
angular
.module('air')
.config(function (datepickerConfig, datepickerPopupConfig) {
  datepickerConfig.showWeeks = false;
  datepickerConfig.dayFormat = 'd';
  datepickerConfig.startingDay = 1;
  datepickerConfig.minDate = new Date();
  datepickerConfig.maxDate = moment().add(18, 'months').toDate();
  datepickerConfig.maxMode = 'month';
  datepickerPopupConfig.toggleWeeksText = null;
  datepickerPopupConfig.showButtonBar = false;
});

// Source: src/html/scripts/air-sales/services/query-parser.js
angular
    .module('air')
    .factory('airQueryCodec',
        /**
         * @ngdoc service
         * @name air.airQueryCodec
         * @memberOf ftl.air
         *
         * @requires $rootScope
         *
         * @description
         * convert air queries from/to stateParmas, and to api Query
         */
        function($http, settings, $log) {
            var knownFlags = [
                'nonstop',
                'direct',
                // No advance purchase fares
                'nap',
                // No penalties fares
                'npe',
                // No restriction fares
                'nr',
                // Refundable fares
                'rf',
                'flexible'
            ];
            var MOMENT_DATE_ISO = 'YYYY-MM-DD';
            var ROUND_TRIP = 2;
            var ONE_WAY = 1;
            var MULTI_SEGMENTS = 3;
            // regexFlight = /^[A-Z]{3},[A-Z]{3},\d{4}-\d{2}-\d{2}$/,
            // this regex stinks
            // regexTravellers = /^\d,\d?\d(,\d?\d){0,5}$/;

            return {
                ROUND_TRIP: ROUND_TRIP,
                ONE_WAY: ONE_WAY,
                MULTI_SEGMENTS: MULTI_SEGMENTS,
                knownFlags: knownFlags,
                newQuery: newQuery,
                fromStateParams: function(encoded) {
                    //console.log('rrrr', encoded);
                    if (!encoded.flight) {
                        return null;
                    }

                    var iataCodes = {};
                    var decoded = newQuery();
                    //console.log('rrrr', decoded);
                    // segments
                    if (!angular.isArray(encoded.flight)) {
                        encoded.flight = [encoded.flight];
                    }
                    decoded.segments = encoded.flight.map(function(flight) {
                        // TO_DO validate regexFlight
                        var bits = flight.split(',');

                        if (!iataCodes[bits[0]]) {
                            iataCodes[bits[0]] = { code: bits[0], name: bits[0] };
                        }
                        if (!iataCodes[bits[1]]) {
                            iataCodes[bits[1]] = { code: bits[1], name: bits[0] };
                        }
                        return {
                            from: iataCodes[bits[0]],
                            to: iataCodes[bits[1]],
                            date: bits[2]
                        };
                    });

                    // linking tripType t segmentLength is a bad idea.
                    // ex: this bugs if its a departure open jaw.
                    decoded.tripType = Math.min(3, decoded.segments.length);
                    // thus...
                    if (decoded.tripType === 2 && decoded.segments[0].from !== decoded.segments[1].to) {
                        decoded.tripType = 3;
                    }

                    // travellers
                    encoded.travellers = encoded.travellers || '1';
                    var travellers = encoded.travellers.split(',');
                    decoded.adults = parseInt(travellers.shift(), 10) || 1;
                    console.log('travellers', travellers);
                    decoded.ages = travellers
                        .map(function(a) {
                            return {
                                v: parseInt(a, 10)
                            };
                        })
                        .filter(function(c) {
                            return c.v >= 0;
                        });
                    decoded.children = decoded.ages.length;

                    // flags
                    if (encoded.flags) {
                        encoded.flags
                            .split(',')
                            .filter(function(value) {
                                return knownFlags.indexOf(value) > -1;
                            })
                            .reduce(function(target, flag) {
                                target.flags[flag] = true;
                                return target;
                            }, decoded);
                    }

                    decoded.cabin = encoded.cabin || '-';
                    decoded.corpid = encoded.corpid;


                    // TODO allow multiple airlines
                    if (encoded.airlines) {
                        var pair = encoded.airlines.split('_');
                        decoded.preferredAirline = { code: pair[0], name: pair[1] };
                    }

                    return $http.get(settings.air.airportsResolve.replace(':codes', Object.keys(iataCodes).join(',')))
                        .then(function(res) {
                            angular.forEach(res.data, function(item) {
                                angular.extend(iataCodes[item.code], item);
                            });
                            $log.debug('decoded query', decoded);
                            return decoded;
                        }, function() {
                            // error just return without resolved names
                            return decoded;
                        });
                },
                toApi: function(query) {
                    var cabin = query.cabin || '-';
                    var out = '';
                    var carriers = query.preferredAirline ? query.preferredAirline.code : 'YY';

                    // ?itinerary=YY..CMNPAR20140619-&itinerary=YY..PARCMN20140628-&flags=direct,flexDates&travellers=2,1,4

                    // exclude VYTB
                    angular.forEach(query.segments, function(segment) {
                        out += '&itinerary=' + carriers + '..' + segment.from.code + segment.to.code + moment(segment.date).format('YYYYMMDD') + cabin;
                    });
                    // Convert travellers.
                    var travellers = query.ages.map(function(age) {
                        return age.v;
                    });
                    travellers.unshift(query.adults);
                    out += '&travellers=' + travellers.join(',');
                    var flags = Object.keys(query.flags)
                        .filter(function(flag) {
                            return query.flags[flag] && knownFlags.indexOf(flag) > -1;
                        }).join(',');

                    if (['M', 'C', 'F'].indexOf(query.cabin) > -1) {
                        out += '&cabin=' + query.cabin;
                    }

                    if (flags) {
                        out += '&flags=' + flags;
                    }

                    if (query.corpid) {
                        out += '&corpid=' + query.corpid;
                    }

                    return out;
                },
                toStateParams: function(decoded) {
                    var encoded = [];
                    console.log('aaaa', decoded);
                    //
                    // Segments
                    //
                    // make sure rountd trip data is set
                    if (decoded.tripType === ROUND_TRIP) {
                        decoded.segments[1].from = decoded.segments[0].to;
                        decoded.segments[1].to = decoded.segments[0].from;
                        // make sure other segments are cut;
                        decoded.segments.length = 2;
                    }
                    // make sure first flight
                    if (decoded.tripType === ONE_WAY) {
                        decoded.segments.length = 1;
                    }
                    // make sure we cuttof garbage.
                    // decoded.segments.length = Math.min(decoded.tripType, decoded.segments.length);

                    angular.forEach(decoded.segments, function(segment) {
                        encoded.push('flight=' +
                            segment.from.code + ',' +
                            segment.to.code + ',' +
                            moment(segment.date).format(MOMENT_DATE_ISO));
                    });

                    //
                    // Travellers
                    //
                    var travellers = (decoded.ages || []).map(function(a) {
                        return a.v;
                    });
                    // add adults in i=0
                    travellers.unshift(decoded.adults);
                    encoded.push('travellers=' + travellers.join(','));

                    //
                    // Cabin
                    //
                    if (decoded.cabin) {
                        encoded.push('&cabin=' + decoded.cabin);
                    }

                    if (decoded.preferredAirline) {
                        encoded.push('&airlines=' + decoded.preferredAirline.code + '_' + decoded.preferredAirline.name);
                    }

                    if (decoded.corpid) {
                        encoded.push('&corpid=' + decoded.corpid);
                    }

                    //
                    // Flags
                    //
                    // console.log('toStateParams flags',decoded.flags);
                    var flags = knownFlags.filter(function(flag) {
                        // console.log('toStateParams flag ',flag,decoded.flags[flag]);

                        // return true if query.flags[flag] === true
                        return decoded.flags[flag];
                    }).join(',');

                    if (flags) {
                        encoded.push('flags=' + flags);
                    }
                    $log.debug('toStateParams', decoded, encoded);

                    return encoded.join('&');
                }
            };

            function newQuery() {
                return {
                    tripType: ROUND_TRIP,
                    segments: [],
                    adults: 1,
                    children: 0,
                    ages: [],
                    cabin: '-',
                    corpid: '',
                    flags: knownFlags.reduce(function(reduced, value) {
                        reduced[value] = false;
                        return reduced;
                    }, {})
                };
            }
        });

// Source: src/html/scripts/air-sales/controllers/search-form.js
angular
	.module("air")
	.controller("air_SearchFormCtrl", function (
		$scope,
		settings,
		$window,
		$http,
		$log,
		airQueryCodec,
		$sce, $document
	) {
		$log.info("air_SearchFormCtrl:airQueryCodec", $scope.query);

		var options = {
			minDeparture: moment()
				.startOf("day")
				.toDate(),
			maxDeparture: moment()
				.add(1, "years")
				.toDate(),
			maxAdults: 9,
			maxChildren: 9,
			maxPax: 9,
			adults: [1, 2, 3, 4, 5, 6, 7, 8, 9],
			children: [0, 1, 2, 3, 4, 5, 7, 8, 9],
			ages: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11],
			tripTypes: [{
				code: 1,
				label: "Direct flight"
			}, {
				code: 2,
				label: "Round trip"
			}, {
				"code": 3,
				"label": "Multi-segments/open-jaw"
			}]
		};
		/** new form with range date picker configuration */
		$scope.date1 = "";
		$scope.date2 = "";

		$scope.roomDiv = false;
		$scope.roomInput = "Adultes - Enfant ...";

		function rangePickerChange() {
			//console.log('-------- date simple format ------', $scope.date1); 
			var date2 = $scope.date1.split('--');
			$scope.query.segments[0].date = convertDate(date2["0"]);
			$scope.query.segments[1].date = convertDate(date2["1"]);
			console.log(' in range picker fubnction ', $scope.query);
		};

		function simplePickerChange() {
			//console.log('-------- date simple format ------', $scope.date2); 
			$scope.query.segments[0].date = convertDate($scope.date2);
			console.log(' in range picker fubnction ', $scope.query);
		};

		function roomChange() {
			var adulteCount = 0;

			$scope.roomInput = "Adultes " + $scope.query.adults + " ,Enfant " + $scope.query.children;
			$scope.roomDiv = $scope.roomDiv ? false : true;
			//console.log("&&&&&&&&&&&&&&============================================ rooms", $scope.roomInput);
		}
		$document.bind('click', function (event) {
			if (event.target.id !== "inputDivRooms" && event.target.className !== "guests__inputs divRooms ng-scope" && $scope.roomDiv === true && !event.target.classList.contains("divRoomsInputs") && !event.target.nextSibling.classList.contains("divRoomsInputs")) {
				//console.log("&event  ", event, event.target.id, event.target.className, $scope.roomDiv);
				$scope.roomDiv = $scope.roomDiv ? false : true;
				$scope.$apply();
			}


		});

		function convertDate(date) {
			var splitDate = date.split('/');
			var convertedDate = new Date(splitDate[2], splitDate[1] - 1, splitDate[0]);
			return convertedDate.toLocaleDateString("en-US");
		}
		/** end of --------new form with range date picker configuration */
		//
		// , {
		//     "code": 3,
		//     "label": "Multi-segments/open-jaw"
		// }
		var calendars = {};
		var query;
		//
		if ($scope.query && !settings.air.standaloneSearchForm) {

			query = angular.copy($scope.query);
			console.log("------- query if existe with setting air -----", query);
		}
		// else if ($scope.query) {

		// 	query = angular.copy($scope.query);
		// 	console.log("------- query if existe only -----", query);
		// } 
		else {

			query = airQueryCodec.newQuery();
			query.tripType = 0;
			setSegmentCount(2);
			console.log("------- query if not  existe -----", query);
		}



		$scope.$watch("query.children", function (fresh, old) {
			if (fresh !== old) {
				while (query.ages.length < fresh) {
					query.ages.push({});
				}
				while (query.ages.length > fresh) {
					query.ages.pop();
				}
			}
		});

		if (!query.tripType) setSegmentCount(2);

		angular.extend($scope, {
			airports: [],
			airlines: [],
			query: query,
			calendars: calendars,
			setTripType: setSegmentCount,
			addSegment: addSegment,
			removeSegment: removeSegment,
			toggleCalendar: toggleCalendar,
			searchAirports: searchAirports,
			searchAirlines: searchAirlines,
			search: doSearch,
			dateChanged: dateChanged,
			maxSegments: 6,
			options: options,
			// HACK for ui-select wich renders html from different origin (cdnjs/apollo)
			trustAsHtml: $sce.trustAsHtml,
			simplePickerChange: simplePickerChange,
			rangePickerChange: rangePickerChange,
			roomChange: roomChange

		});

		/*
		 * @ngdoc method
		 * @name air_SearchFormCtrl#setSegmentCount
		 * @memberOf ftl.air
		 *
		 *
		 * @description
		 * updates segment array in query
		 */
		function setSegmentCount(tripType) {
			var newCount = tripType;
			if (newCount === query.tripType) return;

			var segments = query.segments;
			var previous,
				i = 0;
			// initial state
			console.log("---- length setSegmentCount ----", query);

			if (query.segments.length === 0) {
				segments.push({
					options: {
						minDate: options.minDeparture,
						minMonth: moment(options.minDeparture).startOf("month"),
						initDate: "query.segments[0].options.minMonth"
					}
				});
			}

			i = segments.length;
			while (segments.length < newCount) {
				previous = query.segments[segments.length - 1];
				segments.push({
					from: previous.to,
					options: {
						minDate: previous.date ? previous.date : previous.minDate,
						minMonth: previous.date ? moment(previous.date).startOf("month") : previous.minMonth,
						initDate: "query.segments[" + i + "].options.minMonth"
					}
				});
				i++;
			}
			if (newCount === 2) {
				segments[1].from = segments[0].to;
				segments[1].to = segments[0].from;
			}


			query.tripType = newCount;


		}

		function addSegment() {
			var i = query.segments.length;

			if (i >= 6) {
				return;
			}

			var previous = query.segments[i - 1];
			query.segments.push({
				from: previous.to,
				options: {
					minDate: previous.date ? previous.date : previous.minDate,
					minMonth: previous.date ? moment(previous.date).startOf("month") : previous.minMonth,
					initDate: "query.segments[" + i + "].options.minMonth"
				}
			});
		}

		function removeSegment(segment) {
			var index = query.segments.indexOf(segment);
			if (index > -1) {
				query.segments.splice(index, 1);
			}
		}

		function searchAirports(val) {
			$scope.airports = [];
			if (val.length < 1) {
				return [];
			}

			return $http
				.get(settings.air.airportsSearch.replace(":term", val))
				.then(function (res) {
					$scope.airports = res.data || [];
					return $scope.airports;
				});
		}

		function searchAirlines(val) {
			$scope.airlines = [];
			if (val.length < 1) {
				return [];
			}

			return $http
				.get(settings.air.airlinesSearch.replace(":term", val))
				.then(function (res) {
					$scope.airlines = res.data || [];
					return $scope.airlines;
				});
		}

		function toggleCalendar($event, cal) {
			var key = "cal" + cal;
			var old = calendars[key];
			for (var k in calendars) {
				calendars[k] = false;
			}
			calendars[key] = !old;
		}

		function doSearch() {
			// make sure last point ===  first
			if (query.tripType === airQueryCodec.ROUND_TRIP) {
				query.segments[1].from = query.segments[0].to;
				query.segments[1].to = query.segments[0].from;
			}

			var qs = airQueryCodec.toStateParams(query);
			if (settings.air && settings.air.standaloneSearchForm) {
				$window.location.href = settings.air.salesApp + "?" + qs;
			} else {
				$window.location.href = settings.links.search + "?" + qs;
			}
		}

		/**
		 * watching date changes to update next segment's min departure date
		 */
		function dateChanged(index) {
			var segments = query.segments,
				segment,
				len = query.segments.length,
				minDate,
				initDate;

			if (!segments[index]) return;

			minDate = query.segments[index].date;

			initDate = moment(minDate)
				.startOf("month")
				.toDate();
			index++;

			while (index < len) {
				segment = segments[index];
				segment.options = segments.options || {
					initDate: "$parent.$parent.options.minMonth"
				};
				segment.options.minDate = minDate;
				segment.options.minMonth = moment(minDate)
					.startOf("month")
					.toDate();
				if (segment.date && moment(segment.date).isBefore(minDate)) {
					segment.date = minDate;
				}
				index++;
			}
			//console.log('dateChanged', index);
		}
	});
}());